@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2013 Red Hat, Inc.
 # All Rights Reserved.
 #
@@ -19,10 +21,8 @@ Use gzip compression if the client accepts it.
 
 import re
 
-from oslo_log import log as logging
-
 from glance.common import wsgi
-from glance.i18n import _LI
+import glance.openstack.common.log as logging
 
 LOG = logging.getLogger(__name__)
 
@@ -32,7 +32,7 @@ class GzipMiddleware(wsgi.Middleware):
     re_zip = re.compile(r'\bgzip\b')
 
     def __init__(self, app):
-        LOG.info(_LI("Initialized gzip middleware"))
+        LOG.info(_("Initialized gzip middleware"))
         super(GzipMiddleware, self).__init__(app)
 
     def process_response(self, response):
@@ -48,7 +48,7 @@ class GzipMiddleware(wsgi.Middleware):
             # that they can be compressed without reading
             # the whole content in memory. Notice that using
             # lazy will set response's content-length to 0.
-            content_type = response.headers.get("Content-Type", "")
+            content_type = response.headers["Content-Type"]
             lazy = content_type == "application/octet-stream"
 
             # NOTE(flaper87): Webob takes care of the compression
