@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2010-2012 OpenStack LLC.
+# Copyright 2013 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -38,26 +38,28 @@ from glance.api import policy
 import glance.api.v1
 from glance.api.v1 import controller
 from glance.api.v1 import filters
+from glance.api.v1 import upload_utils
 from glance.common import exception
+from glance.common import property_utils
 from glance.common import utils
 from glance.common import wsgi
 from glance import notifier
 import glance.openstack.common.log as logging
-from glance import registry
+from glance.openstack.common import strutils
+import glance.registry.client.v1.api as registry
 from glance.store import (get_from_backend,
                           get_size_from_backend,
-                          safe_delete_from_backend,
-                          schedule_delayed_delete_from_backend,
                           get_store_from_location,
                           get_store_from_scheme)
 
-CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 SUPPORTED_PARAMS = glance.api.v1.SUPPORTED_PARAMS
 SUPPORTED_FILTERS = glance.api.v1.SUPPORTED_FILTERS
-CONTAINER_FORMATS = ['ami', 'ari', 'aki', 'bare', 'ovf']
-DISK_FORMATS = ['ami', 'ari', 'aki', 'vhd', 'vmdk', 'raw', 'qcow2', 'vdi',
-                'iso']
+ACTIVE_IMMUTABLE = glance.api.v1.ACTIVE_IMMUTABLE
+
+CONF = cfg.CONF
+CONF.import_opt('disk_formats', 'glance.domain')
+CONF.import_opt('container_formats', 'glance.domain')
 
 
 def validate_image_meta(req, values):
@@ -67,12 +69,12 @@ def validate_image_meta(req, values):
     container_format = values.get('container_format')
 
     if 'disk_format' in values:
-        if disk_format not in DISK_FORMATS:
+        if disk_format not in CONF.disk_formats:
             msg = "Invalid disk format '%s' for image." % disk_format
             raise HTTPBadRequest(explanation=msg, request=req)
 
     if 'container_format' in values:
-        if container_format not in CONTAINER_FORMATS:
+        if container_format not in CONF.container_formats:
             msg = "Invalid container format '%s' for image." % container_format
             raise HTTPBadRequest(explanation=msg, request=req)
 
@@ -97,17 +99,18 @@ def validate_image_meta(req, values):
     return values
 
 
-def redact_loc(image_meta):
+def redact_loc(image_meta, copy_dict=True):
     """
     Create a shallow copy of image meta with 'location' removed
     for security (as it can contain credentials).
     """
-    if 'location' in image_meta:
-        tmp_image_meta = copy.copy(image_meta)
-        del tmp_image_meta['location']
-        return tmp_image_meta
-
-    return image_meta
+    if copy_dict:
+        new_image_meta = copy.copy(image_meta)
+    else:
+        new_image_meta = image_meta
+    new_image_meta.pop('location', None)
+    new_image_meta.pop('location_data', None)
+    return new_image_meta
 
 
 class Controller(controller.BaseController):
@@ -134,6 +137,10 @@ class Controller(controller.BaseController):
         registry.configure_registry_client()
         self.policy = policy.Enforcer()
         self.pool = eventlet.GreenPool(size=1024)
+        if property_utils.is_property_protection_enabled():
+            self.prop_enforcer = property_utils.PropertyRules()
+        else:
+            self.prop_enforcer = None
 
     def _enforce(self, req, action):
         """Authorize an action against our policies"""
@@ -142,6 +149,103 @@ class Controller(controller.BaseController):
         except exception.Forbidden:
             raise HTTPForbidden()
 
+    def _enforce_create_protected_props(self, create_props, req):
+        """
+        Check request is permitted to create certain properties
+
+        :param create_props: List of properties to check
+        :param req: The WSGI/Webob Request object
+
+        :raises HTTPForbidden if request forbidden to create a property
+        """
+        if property_utils.is_property_protection_enabled():
+            for key in create_props:
+                if (self.prop_enforcer.check_property_rules(
+                        key, 'create', req.context.roles) is False):
+                    msg = _("Property '%s' is protected" % key)
+                    LOG.debug(msg)
+                    raise HTTPForbidden(explanation=msg,
+                                        request=req,
+                                        content_type="text/plain")
+
+    def _enforce_read_protected_props(self, image_meta, req):
+        """
+        Remove entries from metadata properties if they are read protected
+
+        :param image_meta: Mapping of metadata about image
+        :param req: The WSGI/Webob Request object
+        """
+        if property_utils.is_property_protection_enabled():
+            for key in image_meta['properties'].keys():
+                if (self.prop_enforcer.check_property_rules(
+                        key, 'read', req.context.roles) is False):
+                    image_meta['properties'].pop(key)
+
+    def _enforce_update_protected_props(self, update_props, image_meta,
+                                        orig_meta, req):
+        """
+        Check request is permitted to update certain properties.  Read
+        permission is required to delete a property.
+
+        If the property value is unchanged, i.e. a noop, it is permitted,
+        however, it is important to ensure read access first.  Otherwise the
+        value could be discovered using brute force.
+
+        :param update_props: List of properties to check
+        :param image_meta: Mapping of proposed new metadata about image
+        :param orig_meta: Mapping of existing metadata about image
+        :param req: The WSGI/Webob Request object
+
+        :raises HTTPForbidden if request forbidden to create a property
+        """
+        if property_utils.is_property_protection_enabled():
+            for key in update_props:
+                has_read = self.prop_enforcer.check_property_rules(
+                        key, 'read', req.context.roles)
+                if ((self.prop_enforcer.check_property_rules(
+                        key, 'update', req.context.roles) is False and
+                        image_meta['properties'][key] !=
+                        orig_meta['properties'][key]) or not has_read):
+                    msg = _("Property '%s' is protected" % key)
+                    LOG.debug(msg)
+                    raise HTTPForbidden(explanation=msg,
+                                        request=req,
+                                        content_type="text/plain")
+
+    def _enforce_delete_protected_props(self, delete_props, image_meta,
+                                        orig_meta, req):
+        """
+        Check request is permitted to delete certain properties.  Read
+        permission is required to delete a property.
+
+        Note, the absence of a property in a request does not necessarily
+        indicate a delete.  The requester may not have read access, and so can
+        not know the property exists.  Hence, read access is a requirement for
+        delete, otherwise the delete is ignored transparently.
+
+        :param delete_props: List of properties to check
+        :param image_meta: Mapping of proposed new metadata about image
+        :param orig_meta: Mapping of existing metadata about image
+        :param req: The WSGI/Webob Request object
+
+        :raises HTTPForbidden if request forbidden to create a property
+        """
+        if property_utils.is_property_protection_enabled():
+            for key in delete_props:
+                if (self.prop_enforcer.check_property_rules(
+                        key, 'read', req.context.roles) is False):
+                    # NOTE(bourke): if read protected, re-add to image_meta to
+                    # prevent deletion
+                    image_meta['properties'][key] = \
+                        orig_meta['properties'][key]
+                elif (self.prop_enforcer.check_property_rules(
+                        key, 'delete', req.context.roles) is False):
+                    msg = _("Property '%s' is protected" % key)
+                    LOG.debug(msg)
+                    raise HTTPForbidden(explanation=msg,
+                                        request=req,
+                                        content_type="text/plain")
+
     def index(self, req):
         """
         Returns the following information for all public, available images:
@@ -169,7 +273,7 @@ class Controller(controller.BaseController):
         params = self._get_query_params(req)
         try:
             images = registry.get_images_list(req.context, **params)
-        except exception.Invalid, e:
+        except exception.Invalid as e:
             raise HTTPBadRequest(explanation="%s" % e)
 
         return dict(images=images)
@@ -208,8 +312,9 @@ class Controller(controller.BaseController):
             # to it, however we do not return this potential security
             # information to the API end user...
             for image in images:
-                del image['location']
-        except exception.Invalid, e:
+                redact_loc(image, copy_dict=False)
+                self._enforce_read_protected_props(image, req)
+        except exception.Invalid as e:
             raise HTTPBadRequest(explanation="%s" % e)
         return dict(images=images)
 
@@ -261,7 +366,8 @@ class Controller(controller.BaseController):
         """
         self._enforce(req, 'get_image')
         image_meta = self.get_image_meta_or_404(req, id)
-        del image_meta['location']
+        image_meta = redact_loc(image_meta)
+        self._enforce_read_protected_props(image_meta, req)
         return {
             'image_meta': image_meta
         }
@@ -275,7 +381,7 @@ class Controller(controller.BaseController):
         If the above constraint is violated, we reject with 400 "Bad Request".
         """
         if source:
-            for scheme in ['s3', 'swift', 'http']:
+            for scheme in ['s3', 'swift', 'http', 'rbd', 'sheepdog', 'cinder']:
                 if source.lower().startswith(scheme):
                     return source
             msg = _("External sourcing not supported for store %s") % source
@@ -288,16 +394,19 @@ class Controller(controller.BaseController):
     def _copy_from(req):
         return req.headers.get('x-glance-api-copy-from')
 
-    @staticmethod
-    def _external_source(image_meta, req):
-        source = image_meta.get('location', Controller._copy_from(req))
+    def _external_source(self, image_meta, req):
+        source = image_meta.get('location')
+        if source is not None:
+            self._enforce(req, 'set_image_location')
+        else:
+            source = Controller._copy_from(req)
         return Controller._validate_source(source, req)
 
     @staticmethod
     def _get_from_store(context, where):
         try:
             image_data, image_size = get_from_backend(context, where)
-        except exception.NotFound, e:
+        except exception.NotFound as e:
             raise HTTPNotFound(explanation="%s" % e)
         image_size = int(image_size) if image_size else None
         return image_data, image_size
@@ -316,6 +425,8 @@ class Controller(controller.BaseController):
         self._enforce(req, 'download_image')
         image_meta = self.get_active_image_meta_or_404(req, id)
 
+        self._enforce_read_protected_props(image_meta, req)
+
         if image_meta.get('size') == 0:
             image_iterator = iter([])
         else:
@@ -324,7 +435,7 @@ class Controller(controller.BaseController):
             image_iterator = utils.cooperative_iter(image_iterator)
             image_meta['size'] = size or image_meta['size']
 
-        del image_meta['location']
+        image_meta = redact_loc(image_meta)
         return {
             'image_iterator': image_iterator,
             'image_meta': image_meta,
@@ -374,7 +485,7 @@ class Controller(controller.BaseController):
             raise HTTPConflict(explanation=msg,
                                request=req,
                                content_type="text/plain")
-        except exception.Invalid, e:
+        except exception.Invalid as e:
             msg = (_("Failed to reserve image. Got error: %(e)s") % locals())
             for line in msg.split('\n'):
                 LOG.debug(line)
@@ -408,7 +519,7 @@ class Controller(controller.BaseController):
                 image_data, image_size = self._get_from_store(req.context,
                                                               copy_from)
             except Exception as e:
-                self._safe_kill(req, image_meta['id'])
+                upload_utils.safe_kill(req, image_meta['id'])
                 msg = _("Copy from external source failed: %s") % e
                 LOG.debug(msg)
                 return
@@ -417,7 +528,7 @@ class Controller(controller.BaseController):
             try:
                 req.get_content_type('application/octet-stream')
             except exception.InvalidContentType:
-                self._safe_kill(req, image_meta['id'])
+                upload_utils.safe_kill(req, image_meta['id'])
                 msg = _("Content-Type must be application/octet-stream")
                 LOG.debug(msg)
                 raise HTTPBadRequest(explanation=msg)
@@ -436,98 +547,16 @@ class Controller(controller.BaseController):
         LOG.debug(_("Uploading image data for image %(image_id)s "
                     "to %(scheme)s store"), locals())
 
-        try:
-            self.notifier.info("image.prepare", redact_loc(image_meta))
-            location, size, checksum = store.add(
-                image_meta['id'],
-                utils.CooperativeReader(image_data),
-                image_meta['size'])
+        self.notifier.info("image.prepare", redact_loc(image_meta))
 
-            def _kill_mismatched(image_meta, attr, actual):
-                supplied = image_meta.get(attr)
-                if supplied and supplied != actual:
-                    msg = _("Supplied %(attr)s (%(supplied)s) and "
-                            "%(attr)s generated from uploaded image "
-                            "(%(actual)s) did not match. Setting image "
-                            "status to 'killed'.") % locals()
-                    LOG.error(msg)
-                    self._safe_kill(req, image_id)
-                    self._initiate_deletion(req, location, image_id)
-                    raise HTTPBadRequest(explanation=msg,
-                                         content_type="text/plain",
-                                         request=req)
+        image_meta, location, loc_meta = upload_utils.upload_data_to_store(
+            req, image_meta, image_data, store, self.notifier)
 
-            # Verify any supplied size/checksum value matches size/checksum
-            # returned from store when adding image
-            _kill_mismatched(image_meta, 'size', size)
-            _kill_mismatched(image_meta, 'checksum', checksum)
+        self.notifier.info('image.upload', redact_loc(image_meta))
 
-            # Update the database with the checksum returned
-            # from the backend store
-            LOG.debug(_("Updating image %(image_id)s data. "
-                      "Checksum set to %(checksum)s, size set "
-                      "to %(size)d"), locals())
-            update_data = {'checksum': checksum,
-                           'size': size}
-            image_meta = registry.update_image_metadata(req.context,
-                                                        image_id,
-                                                        update_data)
-            self.notifier.info('image.upload', redact_loc(image_meta))
+        return location, loc_meta
 
-            return location
-
-        except exception.Duplicate, e:
-            msg = _("Attempt to upload duplicate image: %s") % e
-            LOG.debug(msg)
-            self._safe_kill(req, image_id)
-            raise HTTPConflict(explanation=msg, request=req)
-
-        except exception.Forbidden, e:
-            msg = _("Forbidden upload attempt: %s") % e
-            LOG.debug(msg)
-            self._safe_kill(req, image_id)
-            raise HTTPForbidden(explanation=msg,
-                                request=req,
-                                content_type="text/plain")
-
-        except exception.StorageFull, e:
-            msg = _("Image storage media is full: %s") % e
-            LOG.error(msg)
-            self._safe_kill(req, image_id)
-            self.notifier.error('image.upload', msg)
-            raise HTTPRequestEntityTooLarge(explanation=msg, request=req,
-                                            content_type='text/plain')
-
-        except exception.StorageWriteDenied, e:
-            msg = _("Insufficient permissions on image storage media: %s") % e
-            LOG.error(msg)
-            self._safe_kill(req, image_id)
-            self.notifier.error('image.upload', msg)
-            raise HTTPServiceUnavailable(explanation=msg, request=req,
-                                         content_type='text/plain')
-
-        except exception.ImageSizeLimitExceeded, e:
-            msg = _("Denying attempt to upload image larger than %d bytes."
-                    % CONF.image_size_cap)
-            LOG.info(msg)
-            self._safe_kill(req, image_id)
-            raise HTTPBadRequest(explanation=msg, request=req,
-                                 content_type='text/plain')
-
-        except HTTPError, e:
-            self._safe_kill(req, image_id)
-            #NOTE(bcwaldon): Ideally, we would just call 'raise' here,
-            # but something in the above function calls is affecting the
-            # exception context and we must explicitly re-raise the
-            # caught exception.
-            raise e
-
-        except Exception, e:
-            LOG.exception(_("Failed to upload image"))
-            self._safe_kill(req, image_id)
-            raise HTTPInternalServerError(request=req)
-
-    def _activate(self, req, image_id, location):
+    def _activate(self, req, image_id, location, location_metadata=None):
         """
         Sets the image status to `active` and the image's location
         attribute.
@@ -535,10 +564,14 @@ class Controller(controller.BaseController):
         :param req: The WSGI/Webob Request object
         :param image_id: Opaque image identifier
         :param location: Location of where Glance stored this image
+        :param location_metadata: a dictionary of storage specfic information
         """
         image_meta = {}
         image_meta['location'] = location
         image_meta['status'] = 'active'
+        if location_metadata:
+            image_meta['location_data'] = [{'url': location,
+                                            'metadata': location_metadata}]
 
         try:
             image_meta_data = registry.update_image_metadata(req.context,
@@ -547,7 +580,7 @@ class Controller(controller.BaseController):
             self.notifier.info("image.activate", redact_loc(image_meta_data))
             self.notifier.info("image.update", redact_loc(image_meta_data))
             return image_meta_data
-        except exception.Invalid, e:
+        except exception.Invalid as e:
             msg = (_("Failed to activate image. Got error: %(e)s")
                    % locals())
             LOG.debug(msg)
@@ -555,33 +588,6 @@ class Controller(controller.BaseController):
                                  request=req,
                                  content_type="text/plain")
 
-    def _kill(self, req, image_id):
-        """
-        Marks the image status to `killed`.
-
-        :param req: The WSGI/Webob Request object
-        :param image_id: Opaque image identifier
-        """
-        registry.update_image_metadata(req.context, image_id,
-                                       {'status': 'killed'})
-
-    def _safe_kill(self, req, image_id):
-        """
-        Mark image killed without raising exceptions if it fails.
-
-        Since _kill is meant to be called from exceptions handlers, it should
-        not raise itself, rather it should just log its error.
-
-        :param req: The WSGI/Webob Request object
-        :param image_id: Opaque image identifier
-        """
-        try:
-            self._kill(req, image_id)
-        except Exception, e:
-            LOG.error(_("Unable to kill image %(id)s: "
-                        "%(exc)s") % ({'id': image_id,
-                                       'exc': repr(e)}))
-
     def _upload_and_activate(self, req, image_meta):
         """
         Safely uploads the image data in the request payload
@@ -598,8 +604,11 @@ class Controller(controller.BaseController):
         # See: https://bitbucket.org/ianb/webob/
         # issue/12/fix-for-issue-6-broke-chunked-transfer
         req.is_body_readable = True
-        location = self._upload(req, image_meta)
-        return self._activate(req, image_id, location) if location else None
+        location, location_metadata = self._upload(req, image_meta)
+        return self._activate(req,
+                              image_id,
+                              location,
+                              location_metadata) if location else None
 
     def _get_size(self, context, image_meta, location):
         # retrieve the image size from remote store (if not provided)
@@ -607,19 +616,45 @@ class Controller(controller.BaseController):
                                                                   location)
 
     def _handle_source(self, req, image_id, image_meta, image_data):
+        copy_from = self._copy_from(req)
+        location = image_meta.get('location')
+        sources = filter(lambda x: x, (copy_from, location, image_data))
+        if len(sources) >= 2:
+            msg = _("It's invalid to provide multiple image sources.")
+            LOG.debug(msg)
+            raise HTTPBadRequest(explanation=msg,
+                                 request=req,
+                                 content_type="text/plain")
         if image_data:
             image_meta = self._validate_image_for_activation(req,
                                                              image_id,
                                                              image_meta)
             image_meta = self._upload_and_activate(req, image_meta)
-        elif self._copy_from(req):
+        elif copy_from:
             msg = _('Triggering asynchronous copy from external source')
             LOG.info(msg)
             self.pool.spawn_n(self._upload_and_activate, req, image_meta)
         else:
-            location = image_meta.get('location')
             if location:
                 self._validate_image_for_activation(req, image_id, image_meta)
+                image_size_meta = image_meta.get('size')
+                if image_size_meta:
+                    image_size_store = get_size_from_backend(req.context,
+                                                             location)
+                    # NOTE(zhiyan): A returned size of zero usually means
+                    # the driver encountered an error. In this case the
+                    # size provided by the client will be used as-is.
+                    if (image_size_store and
+                            image_size_store != image_size_meta):
+                        msg = _("Provided image size must match the stored "
+                                "image size. (provided size: %(ps)d, "
+                                "stored size: %(ss)d)" %
+                                {"ps": image_size_meta,
+                                 "ss": image_size_store})
+                        LOG.debug(msg)
+                        raise HTTPConflict(explanation=msg,
+                                           request=req,
+                                           content_type="text/plain")
                 image_meta = self._activate(req, image_id, location)
         return image_meta
 
@@ -683,6 +718,11 @@ class Controller(controller.BaseController):
         is_public = image_meta.get('is_public')
         if is_public:
             self._enforce(req, 'publicize_image')
+        if Controller._copy_from(req):
+            self._enforce(req, 'copy_from')
+
+        self._enforce_create_protected_props(image_meta['properties'].keys(),
+                                             req)
 
         image_meta = self._reserve(req, image_meta)
         id = image_meta['id']
@@ -695,7 +735,7 @@ class Controller(controller.BaseController):
 
         # Prevent client from learning the location, as it
         # could contain security credentials
-        image_meta.pop('location', None)
+        image_meta = redact_loc(image_meta)
 
         return {'image_meta': image_meta}
 
@@ -725,6 +765,17 @@ class Controller(controller.BaseController):
                                 request=req,
                                 content_type="text/plain")
 
+        if req.context.is_admin is False:
+            # Once an image is 'active' only an admin can
+            # modify certain core metadata keys
+            for key in ACTIVE_IMMUTABLE:
+                if (orig_status == 'active' and image_meta.get(key) is not None
+                    and image_meta.get(key) != orig_image_meta.get(key)):
+                    msg = _("Forbidden to modify '%s' of active image.") % key
+                    raise HTTPForbidden(explanation=msg,
+                                        request=req,
+                                        content_type="text/plain")
+
         # The default behaviour for a PUT /images/<IMAGE_ID> is to
         # override any properties that were previously set. This, however,
         # leads to a number of issues for the common use case where a caller
@@ -734,7 +785,7 @@ class Controller(controller.BaseController):
         # properties NOT to be purged. However we also disable purging of
         # properties if an image file is being uploaded...
         purge_props = req.headers.get('x-glance-registry-purge-props', True)
-        purge_props = (utils.bool_from_string(purge_props) and
+        purge_props = (strutils.bool_from_string(purge_props) and
                        image_data is None)
 
         if image_data is not None and orig_status != 'queued':
@@ -761,6 +812,20 @@ class Controller(controller.BaseController):
                                  request=req,
                                  content_type="text/plain")
 
+        # ensure requester has permissions to create/update/delete properties
+        # according to property-protections.conf
+        orig_keys = set(orig_image_meta['properties'])
+        new_keys = set(image_meta['properties'])
+        self._enforce_update_protected_props(
+                orig_keys.intersection(new_keys), image_meta,
+                orig_image_meta, req)
+        self._enforce_create_protected_props(
+                new_keys.difference(orig_keys), req)
+        if purge_props:
+            self._enforce_delete_protected_props(
+                    orig_keys.difference(new_keys), image_meta,
+                    orig_image_meta, req)
+
         try:
             if location:
                 image_meta['size'] = self._get_size(req.context, image_meta,
@@ -775,21 +840,21 @@ class Controller(controller.BaseController):
                 image_meta = self._handle_source(req, id, image_meta,
                                                  image_data)
 
-        except exception.Invalid, e:
+        except exception.Invalid as e:
             msg = (_("Failed to update image metadata. Got error: %(e)s")
                    % locals())
             LOG.debug(msg)
             raise HTTPBadRequest(explanation=msg,
                                  request=req,
                                  content_type="text/plain")
-        except exception.NotFound, e:
+        except exception.NotFound as e:
             msg = (_("Failed to find image to update: %(e)s") % locals())
             for line in msg.split('\n'):
                 LOG.info(line)
             raise HTTPNotFound(explanation=msg,
                                request=req,
                                content_type="text/plain")
-        except exception.Forbidden, e:
+        except exception.Forbidden as e:
             msg = (_("Forbidden to update image: %(e)s") % locals())
             for line in msg.split('\n'):
                 LOG.info(line)
@@ -801,17 +866,12 @@ class Controller(controller.BaseController):
 
         # Prevent client from learning the location, as it
         # could contain security credentials
-        image_meta.pop('location', None)
+        image_meta = redact_loc(image_meta)
+
+        self._enforce_read_protected_props(image_meta, req)
 
         return {'image_meta': image_meta}
 
-    @staticmethod
-    def _initiate_deletion(req, location, id):
-        if CONF.delayed_delete:
-            schedule_delayed_delete_from_backend(location, id)
-        else:
-            safe_delete_from_backend(location, req.context, id)
-
     @utils.mutating
     def delete(self, req, id):
         """
@@ -835,11 +895,16 @@ class Controller(controller.BaseController):
                                 request=req,
                                 content_type="text/plain")
 
-        if image['status'] == 'deleted':
-            msg = _("Forbidden to delete a deleted image.")
+        if image['status'] == 'pending_delete':
+            msg = (_("Forbidden to delete a %s image.") % image['status'])
             LOG.debug(msg)
             raise HTTPForbidden(explanation=msg, request=req,
                                 content_type="text/plain")
+        elif image['status'] == 'deleted':
+            msg = _("Image %s not found.") % id
+            LOG.debug(msg)
+            raise HTTPNotFound(explanation=msg, request=req,
+                               content_type="text/plain")
 
         if image['location'] and CONF.delayed_delete:
             status = 'pending_delete'
@@ -858,15 +923,16 @@ class Controller(controller.BaseController):
             # to delete the image if the backend doesn't yet store it.
             # See https://bugs.launchpad.net/glance/+bug/747799
             if image['location']:
-                self._initiate_deletion(req, image['location'], id)
-        except exception.NotFound, e:
+                upload_utils.initiate_deletion(req, image['location'], id,
+                                               CONF.delayed_delete)
+        except exception.NotFound as e:
             msg = (_("Failed to find image to delete: %(e)s") % locals())
             for line in msg.split('\n'):
                 LOG.info(line)
             raise HTTPNotFound(explanation=msg,
                                request=req,
                                content_type="text/plain")
-        except exception.Forbidden, e:
+        except exception.Forbidden as e:
             msg = (_("Forbidden to delete image: %(e)s") % locals())
             for line in msg.split('\n'):
                 LOG.info(line)
