@@ -15,6 +15,7 @@
 
 import webob.exc
 
+import glance.api.common
 import glance.api.policy
 from glance.common import exception
 from glance.common import utils
@@ -49,42 +50,72 @@ class ImageDataController(object):
         try:
             image = image_repo.get(image_id)
             image.status = 'saving'
-            image_repo.save(image)
-            image.set_data(data, size)
-            image_repo.save(image)
-        except ValueError, e:
+            try:
+                image_repo.save(image)
+                image.set_data(data, size)
+                image_repo.save(image)
+            except exception.NotFound as e:
+                msg = (_("Image %s could not be found after upload."
+                       "The image may have been deleted during the upload: %s")
+                       % (image_id, e))
+                LOG.warn(msg)
+                raise webob.exc.HTTPGone(explanation=msg,
+                                         request=req,
+                                         content_type='text/plain')
+
+        except ValueError as e:
             LOG.debug("Cannot save data for image %s: %s", image_id, e)
             raise webob.exc.HTTPBadRequest(explanation=unicode(e))
-        except exception.Duplicate, e:
-            msg = _("Unable to upload duplicate image data for image: %s")
-            LOG.debug(msg % image_id)
+        except exception.Duplicate as e:
+            msg = (_("Unable to upload duplicate image data for image: %s") %
+                   image_id)
+            LOG.debug(msg)
             raise webob.exc.HTTPConflict(explanation=msg, request=req)
 
-        except exception.Forbidden, e:
-            msg = _("Not allowed to upload image data for image %s")
-            LOG.debug(msg % image_id)
+        except exception.Forbidden as e:
+            msg = (_("Not allowed to upload image data for image %s") %
+                   image_id)
+            LOG.debug(msg)
             raise webob.exc.HTTPForbidden(explanation=msg, request=req)
 
-        except exception.NotFound, e:
+        except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=unicode(e))
 
-        except exception.StorageFull, e:
+        except exception.StorageFull as e:
             msg = _("Image storage media is full: %s") % e
             LOG.error(msg)
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
                                                       request=req)
 
-        except exception.StorageWriteDenied, e:
+        except exception.StorageFull as e:
+            msg = _("Image storage media is full: %s") % e
+            LOG.error(msg)
+            raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
+                                                      request=req)
+
+        except exception.StorageQuotaFull as e:
+            msg = _("Image exceeds the storage quota: %s") % e
+            LOG.error(msg)
+            raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
+                                                      request=req)
+
+        except exception.ImageSizeLimitExceeded as e:
+            msg = _("The incoming image is too large: %") % e
+            LOG.error(msg)
+            raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
+                                                      request=req)
+
+        except exception.StorageWriteDenied as e:
             msg = _("Insufficient permissions on image storage media: %s") % e
             LOG.error(msg)
             raise webob.exc.HTTPServiceUnavailable(explanation=msg,
                                                    request=req)
 
-        except webob.exc.HTTPError, e:
+        except webob.exc.HTTPError as e:
             LOG.error(_("Failed to upload image data due to HTTP error"))
             raise
 
-        except Exception, e:
+        except Exception as e:
             LOG.exception(_("Failed to upload image data due to "
                             "internal error"))
             raise
@@ -93,14 +124,14 @@ class ImageDataController(object):
         image_repo = self.gateway.get_repo(req.context)
         try:
             image = image_repo.get(image_id)
+            if not image.locations:
+                reason = _("No image data could be found")
+                raise exception.NotFound(reason)
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=unicode(e))
         except exception.Forbidden as e:
             raise webob.exc.HTTPForbidden(explanation=unicode(e))
 
-        if not image.locations:
-            reason = _("No image data could be found")
-            raise webob.exc.HTTPNotFound(reason)
         return image
 
 
@@ -118,10 +149,13 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
 class ResponseSerializer(wsgi.JSONResponseSerializer):
     def download(self, response, image):
         response.headers['Content-Type'] = 'application/octet-stream'
-        # NOTE(markwash): filesystem store (and maybe others?) cause a problem
-        # with the caching middleware if they are not wrapped in an iterator
-        # very strange
-        response.app_iter = iter(image.get_data())
+        try:
+            # NOTE(markwash): filesystem store (and maybe others?) cause a
+            # problem with the caching middleware if they are not wrapped in
+            # an iterator very strange
+            response.app_iter = iter(image.get_data())
+        except exception.Forbidden as e:
+            raise webob.exc.HTTPForbidden(unicode(e))
         #NOTE(saschpe): "response.app_iter = ..." currently resets Content-MD5
         # (https://github.com/Pylons/webob/issues/86), so it should be set
         # afterwards for the time being.
