@@ -37,6 +37,8 @@ import glance.store
 LOG = logging.getLogger(__name__)
 
 CONF = cfg.CONF
+CONF.import_opt('disk_formats', 'glance.domain')
+CONF.import_opt('container_formats', 'glance.domain')
 
 
 class ImagesController(object):
@@ -111,47 +113,66 @@ class ImagesController(object):
                 change_method = getattr(self, change_method_name)
                 change_method(req, image, change)
 
-            if len(changes) > 0:
+            if changes:
                     image_repo.save(image)
 
         except exception.NotFound:
-            msg = _("Failed to find image %(image_id)s to update" % locals())
+            msg = _("Failed to find image %(image_id)s to update") % locals()
             LOG.info(msg)
             raise webob.exc.HTTPNotFound(explanation=msg)
         except exception.Forbidden as e:
             raise webob.exc.HTTPForbidden(explanation=unicode(e))
+        except exception.StorageQuotaFull as e:
+            msg = (_("Denying attempt to upload image because it exceeds the ."
+                     "quota: %s") % e)
+            LOG.info(msg)
+            raise webob.exc.HTTPRequestEntityTooLarge(
+                explanation=msg, request=req, content_type='text/plain')
 
         return image
 
     def _do_replace(self, req, image, change):
         path = change['path']
+        path_root = path[0]
         value = change['value']
-        if hasattr(image, path):
-            setattr(image, path, value)
-        elif path in image.extra_properties:
-            image.extra_properties[path] = change['value']
+        if path_root == 'locations':
+            self._do_replace_locations(image, value)
         else:
-            msg = _("Property %s does not exist.")
-            raise webob.exc.HTTPConflict(msg % path)
+            if hasattr(image, path_root):
+                setattr(image, path_root, value)
+            elif path_root in image.extra_properties:
+                image.extra_properties[path_root] = value
+            else:
+                msg = _("Property %s does not exist.")
+                raise webob.exc.HTTPConflict(msg % path_root)
 
     def _do_add(self, req, image, change):
         path = change['path']
+        path_root = path[0]
         value = change['value']
-        if hasattr(image, path) or path in image.extra_properties:
-            msg = _("Property %s already present.")
-            raise webob.exc.HTTPConflict(msg % path)
-        image.extra_properties[path] = value
+        if path_root == 'locations':
+            self._do_add_locations(image, path[1], value)
+        else:
+            if (hasattr(image, path_root) or
+                    path_root in image.extra_properties):
+                msg = _("Property %s already present.")
+                raise webob.exc.HTTPConflict(msg % path_root)
+            image.extra_properties[path_root] = value
 
     def _do_remove(self, req, image, change):
         path = change['path']
-        if hasattr(image, path):
-            msg = _("Property %s may not be removed.")
-            raise webob.exc.HTTPForbidden(msg % path)
-        elif path in image.extra_properties:
-            del image.extra_properties[path]
+        path_root = path[0]
+        if path_root == 'locations':
+            self._do_remove_locations(image, path[1])
         else:
-            msg = _("Property %s does not exist.")
-            raise webob.exc.HTTPConflict(msg % path)
+            if hasattr(image, path_root):
+                msg = _("Property %s may not be removed.")
+                raise webob.exc.HTTPForbidden(msg % path_root)
+            elif path_root in image.extra_properties:
+                del image.extra_properties[path_root]
+            else:
+                msg = _("Property %s does not exist.")
+                raise webob.exc.HTTPConflict(msg % path_root)
 
     @utils.mutating
     def delete(self, req, image_id):
@@ -165,7 +186,71 @@ class ImagesController(object):
         except exception.NotFound as e:
             msg = ("Failed to find image %(image_id)s to delete" % locals())
             LOG.info(msg)
-            raise webob.exc.HTTPNotFound()
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+    def _get_locations_op_pos(self, path_pos, max_pos, allow_max):
+        if path_pos is None or max_pos is None:
+            return None
+        pos = max_pos if allow_max else max_pos - 1
+        if path_pos.isdigit():
+            # NOTE(zhiyan): locations index from '1' by client perspective
+            pos = int(path_pos) - 1
+        elif path_pos != '-':
+            return None
+        if (not allow_max) and (pos not in range(max_pos)):
+            return None
+        return pos
+
+    def _do_replace_locations(self, image, value):
+        if len(image.locations) > 0 and len(value) > 0:
+            msg = _("Cannot replace locations from a non-empty "
+                    "list to a non-empty list.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+        if len(value) == 0:
+            # NOTE(zhiyan): this actually deletes the location
+            # from the backend store.
+            del image.locations[:]
+            if image.status == 'active':
+                image.status = 'queued'
+        else:   # NOTE(zhiyan): len(image.locations) == 0
+            try:
+                image.locations = value
+                if image.status == 'queued':
+                    image.status = 'active'
+            except exception.BadStoreUri as bse:
+                raise webob.exc.HTTPBadRequest(explanation=unicode(bse))
+            except ValueError as ve:    # update image status failed.
+                raise webob.exc.HTTPBadRequest(explanation=unicode(ve))
+
+    def _do_add_locations(self, image, path_pos, value):
+        pos = self._get_locations_op_pos(path_pos,
+                                         len(image.locations), True)
+        if pos is None:
+            msg = _("Invalid position for adding a location.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+        try:
+            image.locations.insert(pos, value)
+            if image.status == 'queued':
+                image.status = 'active'
+        except exception.BadStoreUri as bse:
+            raise webob.exc.HTTPBadRequest(explanation=unicode(bse))
+        except ValueError as ve:    # update image status failed.
+            raise webob.exc.HTTPBadRequest(explanation=unicode(ve))
+
+    def _do_remove_locations(self, image, path_pos):
+        pos = self._get_locations_op_pos(path_pos,
+                                         len(image.locations), False)
+        if pos is None:
+            msg = _("Invalid position for removing a location.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+        try:
+            # NOTE(zhiyan): this actually deletes the location
+            # from the backend store.
+            image.locations.pop(pos)
+        except Exception as e:
+            raise webob.exc.HTTPInternalServerError(explanation=unicode(e))
+        if (len(image.locations) == 0) and (image.status == 'active'):
+            image.status = 'queued'
 
 
 class RequestDeserializer(wsgi.JSONRequestDeserializer):
@@ -173,12 +258,13 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
     _disallowed_properties = ['direct_url', 'self', 'file', 'schema']
     _readonly_properties = ['created_at', 'updated_at', 'status', 'checksum',
                             'size', 'direct_url', 'self', 'file', 'schema']
-    _reserved_properties = ['owner', 'is_public', 'location', 'locations',
-                            'deleted', 'deleted_at']
+    _reserved_properties = ['owner', 'is_public', 'location', 'deleted',
+                            'deleted_at']
     _base_properties = ['checksum', 'created_at', 'container_format',
                         'disk_format', 'id', 'min_disk', 'min_ram', 'name',
                         'size', 'status', 'tags', 'updated_at', 'visibility',
                         'protected']
+    _path_depth_limits = {'locations': {'add': 2, 'remove': 2, 'replace': 1}}
 
     def __init__(self, schema=None):
         super(RequestDeserializer, self).__init__()
@@ -258,24 +344,31 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         as "~0".
         """
         self._validate_json_pointer(pointer)
-        return pointer.lstrip('/').replace('~1', '/').replace('~0', '~')
+        ret = []
+        for part in pointer.lstrip('/').split('/'):
+            ret.append(part.replace('~1', '/').replace('~0', '~').strip())
+        return ret
 
     def _validate_json_pointer(self, pointer):
         """ Validate a json pointer.
 
-        We only accept a limited form of json pointers. Specifically, we do
-        not allow multiple levels of indirection, so there can only be one '/'
-        in the pointer, located at the start of the string.
+        We only accept a limited form of json pointers.
         """
         if not pointer.startswith('/'):
-            msg = _('Pointer `%s` does not start with "/".' % pointer)
+            msg = _('Pointer `%s` does not start with "/".') % pointer
             raise webob.exc.HTTPBadRequest(explanation=msg)
-        if '/' in pointer[1:]:
-            msg = _('Pointer `%s` contains more than one "/".' % pointer)
+        if re.search('/\s*?/', pointer[1:]):
+            msg = _('Pointer `%s` contains adjacent "/".') % pointer
             raise webob.exc.HTTPBadRequest(explanation=msg)
-        if re.match('~[^01]', pointer):
+        if len(pointer) > 1 and pointer.endswith('/'):
+            msg = _('Pointer `%s` end with "/".') % pointer
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+        if pointer[1:].strip() == '/':
+            msg = _('Pointer `%s` does not contains valid token.') % pointer
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+        if re.search('~[^01]', pointer) or pointer.endswith('~'):
             msg = _('Pointer `%s` contains "~" not part of'
-                    ' a recognized escape sequence.' % pointer)
+                    ' a recognized escape sequence.') % pointer
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
     def _get_change_value(self, raw_change, op):
@@ -285,22 +378,44 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         return raw_change['value']
 
     def _validate_change(self, change):
-        path = change['path']
-        if path in self._readonly_properties:
-            msg = "Attribute \'%s\' is read-only." % path
+        path_root = change['path'][0]
+        if path_root in self._readonly_properties:
+            msg = "Attribute \'%s\' is read-only." % path_root
             raise webob.exc.HTTPForbidden(explanation=unicode(msg))
-        if path in self._reserved_properties:
-            msg = "Attribute \'%s\' is reserved." % path
+        if path_root in self._reserved_properties:
+            msg = "Attribute \'%s\' is reserved." % path_root
             raise webob.exc.HTTPForbidden(explanation=unicode(msg))
 
         if change['op'] == 'delete':
             return
 
-        partial_image = {change['path']: change['value']}
-        try:
-            self.schema.validate(partial_image)
-        except exception.InvalidObject as e:
-            raise webob.exc.HTTPBadRequest(explanation=unicode(e))
+        partial_image = None
+        if len(change['path']) == 1:
+            partial_image = {path_root: change['value']}
+        elif ((path_root in _get_base_properties().keys()) and
+              (_get_base_properties()[path_root].get('type', '') == 'array')):
+            # NOTE(zhiyan): cient can use PATCH API to adding element to
+            # the image's existing set property directly.
+            # Such as: 1. using '/locations/N' path to adding a location
+            #             to the image's 'locations' list at N position.
+            #             (implemented)
+            #          2. using '/tags/-' path to appending a tag to the
+            #             image's 'tags' list at last. (Not implemented)
+            partial_image = {path_root: [change['value']]}
+
+        if partial_image:
+            try:
+                self.schema.validate(partial_image)
+            except exception.InvalidObject as e:
+                raise webob.exc.HTTPBadRequest(explanation=unicode(e))
+
+    def _validate_path(self, op, path):
+        path_root = path[0]
+        limits = self._path_depth_limits.get(path_root, {})
+        if len(path) != limits.get(op, 1):
+            msg = _("Invalid JSON pointer for this resource: "
+                    "\'/%s\'") % '/'.join(path)
+            raise webob.exc.HTTPBadRequest(explanation=unicode(msg))
 
     def _parse_json_schema_change(self, raw_change, draft_version):
         if draft_version == 10:
@@ -313,8 +428,8 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
             msg = _('Unrecognized JSON Schema draft version')
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
-        path = self._decode_json_pointer(path)
-        return op, path
+        path_list = self._decode_json_pointer(path)
+        return op, path_list
 
     def update(self, request):
         changes = []
@@ -342,6 +457,8 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
             (op, path) = self._parse_json_schema_change(raw_change,
                                                         json_schema_version)
 
+            # NOTE(zhiyan): the 'path' is a list.
+            self._validate_path(op, path)
             change = {'op': op, 'path': path}
 
             if not op == 'remove':
@@ -367,14 +484,14 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
 
     def _validate_sort_dir(self, sort_dir):
         if sort_dir not in ['asc', 'desc']:
-            msg = _('Invalid sort direction: %s' % sort_dir)
+            msg = _('Invalid sort direction: %s') % sort_dir
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
         return sort_dir
 
     def _validate_member_status(self, member_status):
         if member_status not in ['pending', 'accepted', 'rejected', 'all']:
-            msg = _('Invalid status: %s' % member_status)
+            msg = _('Invalid status: %s') % member_status
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
         return member_status
@@ -394,6 +511,14 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         marker = params.pop('marker', None)
         sort_dir = params.pop('sort_dir', 'desc')
         member_status = params.pop('member_status', 'accepted')
+
+        # NOTE (flwang) To avoid using comma or any predefined chars to split
+        # multiple tags, now we allow user specify multiple 'tag' parameters
+        # in URL, such as v2/images?tag=x86&tag=64bit.
+        tags = []
+        while 'tag' in params:
+            tags.append(params.pop('tag').strip())
+
         query_params = {
             'sort_key': params.pop('sort_key', 'created_at'),
             'sort_dir': self._validate_sort_dir(sort_dir),
@@ -407,6 +532,9 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         if limit is not None:
             query_params['limit'] = self._validate_limit(limit)
 
+        if tags:
+            query_params['filters']['tags'] = tags
+
         return query_params
 
 
@@ -422,22 +550,37 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
         return base_href
 
     def _format_image(self, image):
-        image_view = dict(image.extra_properties)
-        attributes = ['name', 'disk_format', 'container_format', 'visibility',
-                      'size', 'status', 'checksum', 'protected',
-                      'min_ram', 'min_disk']
-        for key in attributes:
-            image_view[key] = getattr(image, key)
-        image_view['id'] = image.image_id
-        image_view['created_at'] = timeutils.isotime(image.created_at)
-        image_view['updated_at'] = timeutils.isotime(image.updated_at)
-        if CONF.show_image_direct_url and image.locations:  # domain
-            image_view['direct_url'] = image.locations[0]
-        image_view['tags'] = list(image.tags)
-        image_view['self'] = self._get_image_href(image)
-        image_view['file'] = self._get_image_href(image, 'file')
-        image_view['schema'] = '/v2/schemas/image'
-        image_view = self.schema.filter(image_view)  # domain
+        image_view = dict()
+        try:
+            image_view = dict(image.extra_properties)
+            attributes = ['name', 'disk_format', 'container_format',
+                          'visibility', 'size', 'status', 'checksum',
+                          'protected', 'min_ram', 'min_disk']
+            for key in attributes:
+                image_view[key] = getattr(image, key)
+            image_view['id'] = image.image_id
+            image_view['created_at'] = timeutils.isotime(image.created_at)
+            image_view['updated_at'] = timeutils.isotime(image.updated_at)
+
+            if CONF.show_multiple_locations:
+                if image.locations:
+                    image_view['locations'] = list(image.locations)
+                else:
+                    # NOTE (flwang): We will still show "locations": [] if
+                    # image.locations is None to indicate it's allowed to show
+                    # locations but it's just non-existent.
+                    image_view['locations'] = []
+
+            if CONF.show_image_direct_url and image.locations:
+                image_view['direct_url'] = image.locations[0]['url']
+
+            image_view['tags'] = list(image.tags)
+            image_view['self'] = self._get_image_href(image)
+            image_view['file'] = self._get_image_href(image, 'file')
+            image_view['schema'] = '/v2/schemas/image'
+            image_view = self.schema.filter(image_view)  # domain
+        except exception.Forbidden as e:
+            raise webob.exc.HTTPForbidden(unicode(e))
         return image_view
 
     def create(self, response, image):
@@ -479,105 +622,136 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
         response.status_int = 204
 
 
-_BASE_PROPERTIES = {
-    'id': {
-        'type': 'string',
-        'description': _('An identifier for the image'),
-        'pattern': ('^([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-([0-9a-fA-F]){4}'
-                    '-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}$'),
-    },
-    'name': {
-        'type': 'string',
-        'description': _('Descriptive name for the image'),
-        'maxLength': 255,
-    },
-    'status': {
-        'type': 'string',
-        'description': _('Status of the image'),
-        'enum': ['queued', 'saving', 'active', 'killed',
-                 'deleted', 'pending_delete'],
-    },
-    'visibility': {
-        'type': 'string',
-        'description': _('Scope of image accessibility'),
-        'enum': ['public', 'private'],
-    },
-    'protected': {
-        'type': 'boolean',
-        'description': _('If true, image will not be deletable.'),
-    },
-    'checksum': {
-        'type': 'string',
-        'description': _('md5 hash of image contents.'),
-        'type': 'string',
-        'maxLength': 32,
-    },
-    'size': {
-        'type': 'integer',
-        'description': _('Size of image file in bytes'),
-    },
-    'container_format': {
-        'type': 'string',
-        'description': _(''),
-        'type': 'string',
-        'enum': ['bare', 'ovf', 'ami', 'aki', 'ari'],
-    },
-    'disk_format': {
-        'type': 'string',
-        'description': _(''),
-        'type': 'string',
-        'enum': ['raw', 'vhd', 'vmdk', 'vdi', 'iso', 'qcow2',
-                 'aki', 'ari', 'ami'],
-    },
-    'created_at': {
-        'type': 'string',
-        'description': _('Date and time of image registration'),
-        #TODO(bcwaldon): our jsonschema library doesn't seem to like the
-        # format attribute, figure out why!
-        #'format': 'date-time',
-    },
-    'updated_at': {
-        'type': 'string',
-        'description': _('Date and time of the last image modification'),
-        #'format': 'date-time',
-    },
-    'tags': {
-        'type': 'array',
-        'description': _('List of strings related to the image'),
-        'items': {
+def _get_base_properties():
+    return {
+        'id': {
             'type': 'string',
+            'description': _('An identifier for the image'),
+            'pattern': ('^([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-([0-9a-fA-F]){4}'
+                        '-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}$'),
+        },
+        'name': {
+            'type': 'string',
+            'description': _('Descriptive name for the image'),
             'maxLength': 255,
         },
-    },
-    'direct_url': {
-        'type': 'string',
-        'description': _('URL to access the image file kept in external '
-                         'store'),
-    },
-    'min_ram': {
-        'type': 'integer',
-        'description': _('Amount of ram (in MB) required to boot image.'),
-    },
-    'min_disk': {
-        'type': 'integer',
-        'description': _('Amount of disk space (in GB) required to boot '
-                         'image.'),
-    },
-    'self': {'type': 'string'},
-    'file': {'type': 'string'},
-    'schema': {'type': 'string'},
-}
+        'status': {
+            'type': 'string',
+            'description': _('Status of the image (READ-ONLY)'),
+            'enum': ['queued', 'saving', 'active', 'killed',
+                     'deleted', 'pending_delete'],
+        },
+        'visibility': {
+            'type': 'string',
+            'description': _('Scope of image accessibility'),
+            'enum': ['public', 'private'],
+        },
+        'protected': {
+            'type': 'boolean',
+            'description': _('If true, image will not be deletable.'),
+        },
+        'checksum': {
+            'type': 'string',
+            'description': _('md5 hash of image contents. (READ-ONLY)'),
+            'type': 'string',
+            'maxLength': 32,
+        },
+        'size': {
+            'type': 'integer',
+            'description': _('Size of image file in bytes (READ-ONLY)'),
+        },
+        'container_format': {
+            'type': 'string',
+            'description': _('Format of the container'),
+            'type': 'string',
+            'enum': CONF.container_formats,
+        },
+        'disk_format': {
+            'type': 'string',
+            'description': _('Format of the disk'),
+            'type': 'string',
+            'enum': CONF.disk_formats,
+        },
+        'created_at': {
+            'type': 'string',
+            'description': _('Date and time of image registration'
+                             ' (READ-ONLY)'),
+            #TODO(bcwaldon): our jsonschema library doesn't seem to like the
+            # format attribute, figure out why!
+            #'format': 'date-time',
+        },
+        'updated_at': {
+            'type': 'string',
+            'description': _('Date and time of the last image modification'
+                             ' (READ-ONLY)'),
+            #'format': 'date-time',
+        },
+        'tags': {
+            'type': 'array',
+            'description': _('List of strings related to the image'),
+            'items': {
+                'type': 'string',
+                'maxLength': 255,
+            },
+        },
+        'direct_url': {
+            'type': 'string',
+            'description': _('URL to access the image file kept in external '
+                             'store (READ-ONLY)'),
+        },
+        'min_ram': {
+            'type': 'integer',
+            'description': _('Amount of ram (in MB) required to boot image.'),
+        },
+        'min_disk': {
+            'type': 'integer',
+            'description': _('Amount of disk space (in GB) required to boot '
+                             'image.'),
+        },
+        'self': {
+            'type': 'string',
+            'description': '(READ-ONLY)'
+        },
+        'file': {
+            'type': 'string',
+            'description': '(READ-ONLY)'
+        },
+        'schema': {
+            'type': 'string',
+            'description': '(READ-ONLY)'
+        },
+        'locations': {
+            'type': 'array',
+            'items': {
+                'type': 'object',
+                'properties': {
+                    'url': {
+                        'type': 'string',
+                        'maxLength': 255,
+                    },
+                    'metadata': {
+                        'type': 'object',
+                    },
+                },
+                'required': ['url', 'metadata'],
+            },
+            'description': _('A set of URLs to access the image file kept in '
+                             'external store'),
+        },
+    }
 
-_BASE_LINKS = [
-    {'rel': 'self', 'href': '{self}'},
-    {'rel': 'enclosure', 'href': '{file}'},
-    {'rel': 'describedby', 'href': '{schema}'},
-]
+
+def _get_base_links():
+    return [
+        {'rel': 'self', 'href': '{self}'},
+        {'rel': 'enclosure', 'href': '{file}'},
+        {'rel': 'describedby', 'href': '{schema}'},
+    ]
 
 
 def get_schema(custom_properties=None):
-    properties = copy.deepcopy(_BASE_PROPERTIES)
-    links = copy.deepcopy(_BASE_LINKS)
+    properties = _get_base_properties()
+    links = _get_base_links()
     if CONF.allow_additional_image_properties:
         schema = glance.schema.PermissiveSchema('image', properties, links)
     else:
