@@ -1,4 +1,6 @@
-# Copyright 2013 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2013 OpenStack LLC.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,6 +15,5 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from glance import i18n
-
-i18n.enable_lazy()
+from glance.openstack.common import gettextutils
+gettextutils.install('glance')
