@@ -1,6 +1,8 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
-# Copyright 2011 OpenStack Foundation
+# Copyright 2011 OpenStack LLC.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -24,8 +26,6 @@ This is meant to be run as a periodic task, perhaps every half-hour.
 import os
 import sys
 
-from oslo_log import log as logging
-
 # If ../glance/__init__.py exists, add ../ to Python search path, so that
 # it will override what happens to be installed in /usr/(local/)lib/python...
 possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
@@ -36,16 +36,13 @@ if os.path.exists(os.path.join(possible_topdir, 'glance', '__init__.py')):
 
 from glance.common import config
 from glance.image_cache import pruner
-
-CONF = config.CONF
-logging.register_options(CONF)
-CONF.set_default(name='use_stderr', default=True, enforce_type=True)
+from glance.openstack.common import log
 
 
 def main():
     try:
         config.parse_cache_args()
-        logging.setup(CONF, 'glance')
+        log.setup('glance')
 
         app = pruner.Pruner()
         app.run()
