@@ -1,4 +1,6 @@
-# Copyright (c) 2011 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright (c) 2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +20,10 @@ Helper script for starting/stopping/reloading Glance server programs.
 Thanks for some of the code, Swifties ;)
 """
 
+from __future__ import with_statement
+
 import argparse
+import errno
 import fcntl
 import os
 import resource
@@ -36,33 +41,30 @@ possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
 if os.path.exists(os.path.join(possible_topdir, 'glance', '__init__.py')):
     sys.path.insert(0, possible_topdir)
 
-from oslo_config import cfg
-from oslo_utils import units
+from oslo.config import cfg
 
 from glance.common import config
-from glance.i18n import _
 
 CONF = cfg.CONF
 
 ALL_COMMANDS = ['start', 'status', 'stop', 'shutdown', 'restart',
                 'reload', 'force-reload']
-ALL_SERVERS = ['api', 'scrubber']
-RELOAD_SERVERS = ['glance-api']
-GRACEFUL_SHUTDOWN_SERVERS = ['glance-api', 'glance-scrubber']
+ALL_SERVERS = ['api', 'registry', 'scrubber']
+GRACEFUL_SHUTDOWN_SERVERS = ['glance-api', 'glance-registry',
+                             'glance-scrubber']
 MAX_DESCRIPTORS = 32768
-MAX_MEMORY = 2 * units.Gi  # 2 GB
+MAX_MEMORY = (1024 * 1024 * 1024) * 2  # 2 GB
 USAGE = """%(prog)s [options] <SERVER> <COMMAND> [CONFPATH]
 
 Where <SERVER> is one of:
 
-    all, {0}
+    all, api, registry, scrubber
 
 And command is one of:
 
-    {1}
+    start, stop, shutdown, restart, reload, force-reload
 
-And CONFPATH is the optional configuration file to use.""".format(
-    ', '.join(ALL_SERVERS), ', '.join(ALL_COMMANDS))
+And CONFPATH is the optional configuration file to use."""
 
 exitcode = 0
 
@@ -95,11 +97,11 @@ def do_start(verb, pid_file, server, args):
     if verb != 'Respawn' and pid_file == CONF.pid_file:
         for pid_file, pid in pid_files(server, pid_file):
             if os.path.exists('/proc/%s' % pid):
-                print(_("%(serv)s appears to already be running: %(pid)s") %
-                      {'serv': server, 'pid': pid_file})
+                print ("%s appears to already be running: %s" %
+                       (server, pid_file))
                 return
             else:
-                print(_("Removing stale pid file %s") % pid_file)
+                print "Removing stale pid file %s" % pid_file
                 os.unlink(pid_file)
 
         try:
@@ -108,13 +110,14 @@ def do_start(verb, pid_file, server, args):
             resource.setrlimit(resource.RLIMIT_DATA,
                                (MAX_MEMORY, MAX_MEMORY))
         except ValueError:
-            print(_('Unable to increase file descriptor limit.  '
-                    'Running as non-root?'))
+            action = 'increase file descriptor limit'
+            print 'Unable to %s.  Running as non-root?' % action
         os.environ['PYTHON_EGG_CACHE'] = '/tmp'
 
     def write_pid_file(pid_file, pid):
-        with open(pid_file, 'w') as fp:
-            fp.write('%d\n' % pid)
+        fp = open(pid_file, 'w')
+        fp.write('%d\n' % pid)
+        fp.close()
 
     def redirect_to_null(fds):
         with open(os.devnull, 'r+b') as nullfile:
@@ -125,9 +128,9 @@ def do_start(verb, pid_file, server, args):
                     pass
 
     def redirect_to_syslog(fds, server):
-        log_cmd = 'logger'
-        log_cmd_params = '-t "%s[%d]"' % (server, os.getpid())
-        process = subprocess.Popen([log_cmd, log_cmd_params],
+        log_cmd = 'logger -t "%s[%d]"' % (server, os.getpid())
+        process = subprocess.Popen(log_cmd,
+                                   shell=True,
                                    stdin=subprocess.PIPE)
         for desc in fds:  # pipe to logger command
             try:
@@ -153,13 +156,11 @@ def do_start(verb, pid_file, server, args):
 
     def launch(pid_file, conf_file=None, capture_output=False, await_time=0):
         args = [server]
+        print '%sing %s' % (verb, server),
         if conf_file:
             args += ['--config-file', conf_file]
-            msg = (_('%(verb)sing %(serv)s with %(conf)s') %
-                   {'verb': verb, 'serv': server, 'conf': conf_file})
-        else:
-            msg = (_('%(verb)sing %(serv)s') % {'verb': verb, 'serv': server})
-        print(msg)
+            print 'with %s' % conf_file,
+        print
 
         close_stdio_on_exec()
 
@@ -170,8 +171,7 @@ def do_start(verb, pid_file, server, args):
             try:
                 os.execlp('%s' % server, *args)
             except OSError as e:
-                msg = (_('unable to launch %(serv)s. Got error: %(e)s') %
-                       {'serv': server, 'e': e})
+                msg = 'unable to launch %s. Got error: %s' % (server, e)
                 sys.exit(msg)
             sys.exit(0)
         else:
@@ -199,12 +199,10 @@ def do_start(verb, pid_file, server, args):
 
 def do_check_status(pid_file, server):
     if os.path.exists(pid_file):
-        with open(pid_file, 'r') as pidfile:
-            pid = pidfile.read().strip()
-        print(_("%(serv)s (pid %(pid)s) is running...") %
-              {'serv': server, 'pid': pid})
+        pid = open(pid_file).read().strip()
+        print "%s running: %s" % (server, pid)
     else:
-        print(_("%s is stopped") % server)
+        print "No %s running" % server
 
 
 def get_pid_file(server, pid_file):
@@ -220,43 +218,16 @@ def get_pid_file(server, pid_file):
 
     if not os.access(dir, os.W_OK):
         fallback = os.path.join(tempfile.mkdtemp(), '%s.pid' % server)
-        msg = (_('Unable to create pid file %(pid)s.  Running as non-root?\n'
-                 'Falling back to a temp file, you can stop %(service)s '
-                 'service using:\n'
-                 '  %(file)s %(server)s stop --pid-file %(fb)s') %
-               {'pid': pid_file,
-                'service': server,
-                'file': __file__,
-                'server': server,
-                'fb': fallback})
-        print(msg)
+        msg = ('Unable to create pid file %s.  Running as non-root?\n'
+               'Falling back to a temp file, you can stop %s service using:\n'
+               '  %s %s stop --pid-file %s'
+               % (pid_file, server, __file__, server, fallback))
+        print msg
         pid_file = fallback
 
     return pid_file
 
 
-def do_reload(pid_file, server):
-    if server not in RELOAD_SERVERS:
-        msg = (_('Reload of %(serv)s not supported') % {'serv': server})
-        sys.exit(msg)
-
-    pid = None
-    if os.path.exists(pid_file):
-        with open(pid_file, 'r') as pidfile:
-            pid = int(pidfile.read().strip())
-    else:
-        msg = (_('Server %(serv)s is stopped') % {'serv': server})
-        sys.exit(msg)
-
-    sig = signal.SIGHUP
-    try:
-        print(_('Reloading %(serv)s (pid %(pid)s) with signal(%(sig)s)')
-              % {'serv': server, 'pid': pid, 'sig': sig})
-        os.kill(pid, sig)
-    except OSError:
-        print(_("Process %d not running") % pid)
-
-
 def do_stop(server, args, graceful=False):
     if graceful and server in GRACEFUL_SHUTDOWN_SERVERS:
         sig = signal.SIGHUP
@@ -272,21 +243,20 @@ def do_stop(server, args, graceful=False):
         except OSError:
             pass
         try:
-            print(_('Stopping %(serv)s (pid %(pid)s) with signal(%(sig)s)')
-                  % {'serv': server, 'pid': pid, 'sig': sig})
+            print 'Stopping %s  pid: %s  signal: %s' % (server, pid, sig)
             os.kill(pid, sig)
         except OSError:
-            print(_("Process %d not running") % pid)
+            print "Process %d not running" % pid
     for pid_file, pid in pfiles:
-        for _junk in range(150):  # 15 seconds
+        for _junk in xrange(150):  # 15 seconds
             if not os.path.exists('/proc/%s' % pid):
                 break
             time.sleep(0.1)
         else:
-            print(_('Waited 15 seconds for pid %(pid)s (%(file)s) to die;'
-                    ' giving up') % {'pid': pid, 'file': pid_file})
+            print ('Waited 15 seconds for pid %s (%s) to die; giving up' %
+                   (pid, pid_file))
     if not did_anything:
-        print(_('%s is already stopped') % server)
+        print 'No %s running' % server
 
 
 def add_command_parsers(subparsers):
@@ -313,27 +283,27 @@ def main():
     global exitcode
 
     opts = [
-        cfg.SubCommandOpt('server',
-                          title='Server types',
-                          help='Available server types',
-                          handler=add_command_parsers),
-        cfg.StrOpt('pid-file',
-                   metavar='PATH',
-                   help='File to use as pid file. Default: '
-                   '/var/run/glance/$server.pid.'),
-        cfg.IntOpt('await-child',
-                   metavar='DELAY',
-                   default=0,
-                   help='Period to wait for service death '
-                        'in order to report exit code '
-                        '(default is to not wait at all).'),
-        cfg.BoolOpt('capture-output',
-                    default=False,
-                    help='Capture stdout/err in syslog '
-                    'instead of discarding it.'),
-        cfg.BoolOpt('respawn',
-                    default=False,
-                    help='Restart service on unexpected death.'),
+            cfg.SubCommandOpt('server',
+                              title='Server types',
+                              help='Available server types',
+                              handler=add_command_parsers),
+            cfg.StrOpt('pid-file',
+                       metavar='PATH',
+                       help='File to use as pid file. Default: '
+                       '/var/run/glance/$server.pid'),
+            cfg.IntOpt('await-child',
+                       metavar='DELAY',
+                       default=0,
+                       help='Period to wait for service death '
+                            'in order to report exit code '
+                            '(default is to not wait at all)'),
+            cfg.BoolOpt('capture-output',
+                        default=False,
+                        help='Capture stdout/err in syslog '
+                        'instead of discarding'),
+            cfg.BoolOpt('respawn',
+                        default=False,
+                        help='Restart service on unexpected death'),
     ]
     CONF.register_cli_opts(opts)
 
@@ -363,8 +333,7 @@ def main():
                     children[new_pid] = args
                 else:
                     rsn = 'bouncing' if bouncing else 'deliberately stopped'
-                    print(_('Suppressed respawn as %(serv)s was %(rsn)s.')
-                          % {'serv': server, 'rsn': rsn})
+                    print 'Supressed respawn as %s was %s.' % (server, rsn)
 
     if CONF.server.command == 'start':
         children = {}
@@ -396,9 +365,11 @@ def main():
             pid_file = get_pid_file(server, CONF.pid_file)
             do_start('Restart', pid_file, server, CONF.server.args)
 
-    if CONF.server.command in ('reload', 'force-reload'):
+    if (CONF.server.command == 'reload' or
+        CONF.server.command == 'force-reload'):
         for server in CONF.server.servers:
+            do_stop(server, CONF.server.args, graceful=True)
             pid_file = get_pid_file(server, CONF.pid_file)
-            do_reload(pid_file, server)
+            do_start('Restart', pid_file, server, CONF.server.args)
 
     sys.exit(exitcode)
