@@ -57,9 +57,18 @@ common_opts = [
                 help=_('Whether to include the backend image storage location '
                 'in image properties. Revealing storage location can be a '
                 'security risk, so use this setting with caution!')),
+    cfg.BoolOpt('show_multiple_locations', default=False,
+                help=_('Whether to include the backend image locations '
+                       'in image properties. Revealing storage location can '
+                       'be a security risk, so use this setting with '
+                       'caution!  The overrides show_image_direct_url.')),
     cfg.IntOpt('image_size_cap', default=1099511627776,
                help=_("Maximum size of image a user can upload in bytes. "
                       "Defaults to 1099511627776 bytes (1 TB).")),
+    cfg.IntOpt('user_storage_quota', default=0,
+               help=_("Set a system wide quota for every user.  This value is "
+                      "the total number of bytes that a user can use across "
+                      "all storage systems.  A value of 0 means unlimited.")),
     cfg.BoolOpt('enable_v1_api', default=True,
                 help=_("Deploy the v1 OpenStack Images API. ")),
     cfg.BoolOpt('enable_v2_api', default=True,
@@ -148,12 +157,16 @@ def setup_logging():
     root_logger.addHandler(handler)
 
 
-def _get_deployment_flavor():
+def _get_deployment_flavor(flavor=None):
     """
     Retrieve the paste_deploy.flavor config item, formatted appropriately
     for appending to the application name.
+
+    :param flavor: if specified, use this setting rather than the
+                   paste_deploy.flavor configuration setting
     """
-    flavor = CONF.paste_deploy.flavor
+    if not flavor:
+        flavor = CONF.paste_deploy.flavor
     return '' if not flavor else ('-' + flavor)
 
 
@@ -165,7 +178,7 @@ def _get_paste_config_path():
         # to the last config file
         path = CONF.config_file[-1].replace(conf_suffix, paste_suffix)
     else:
-        path = CONF.prog + '-paste.ini'
+        path = CONF.prog + paste_suffix
     return CONF.find_file(os.path.basename(path))
 
 
@@ -183,26 +196,26 @@ def _get_deployment_config_file():
     return os.path.abspath(path)
 
 
-def load_paste_app(app_name=None):
+def load_paste_app(app_name, flavor=None, conf_file=None):
     """
     Builds and returns a WSGI app from a paste config file.
 
     We assume the last config file specified in the supplied ConfigOpts
-    object is the paste config file.
+    object is the paste config file, if conf_file is None.
 
     :param app_name: name of the application to load
+    :param flavor: name of the variant of the application to load
+    :param conf_file: path to the paste config file
 
     :raises RuntimeError when config file cannot be located or application
             cannot be loaded from config file
     """
-    if app_name is None:
-        app_name = CONF.prog
-
     # append the deployment flavor to the application name,
     # in order to identify the appropriate paste pipeline
-    app_name += _get_deployment_flavor()
+    app_name += _get_deployment_flavor(flavor)
 
-    conf_file = _get_deployment_config_file()
+    if not conf_file:
+        conf_file = _get_deployment_config_file()
 
     try:
         logger = logging.getLogger(__name__)
@@ -216,7 +229,7 @@ def load_paste_app(app_name=None):
             CONF.log_opt_values(logger, logging.DEBUG)
 
         return app
-    except (LookupError, ImportError), e:
+    except (LookupError, ImportError) as e:
         msg = _("Unable to load %(app_name)s from "
                 "configuration file %(conf_file)s."
                 "\nGot: %(e)r") % locals()
