@@ -20,6 +20,8 @@
 
 import urlparse
 
+_FATAL_EXCEPTION_FORMAT_ERRORS = False
+
 
 class RedirectException(Exception):
     def __init__(self, url):
@@ -42,8 +44,12 @@ class GlanceException(Exception):
         try:
             message = message % kwargs
         except Exception:
-            # at least get the core message out if something happened
-            pass
+            if _FATAL_EXCEPTION_FORMAT_ERRORS:
+                raise
+            else:
+                # at least get the core message out if something happened
+                pass
+
         super(GlanceException, self).__init__(message)
 
 
@@ -80,6 +86,11 @@ class StorageFull(GlanceException):
     message = _("There is not enough disk space on the image storage media.")
 
 
+class StorageQuotaFull(GlanceException):
+    message = _("The size of the data %(image_size)s will exceed the limit. "
+                "%(remaining)s bytes remaining.")
+
+
 class StorageWriteDenied(GlanceException):
     message = _("Permission to write image storage media denied.")
 
@@ -108,6 +119,10 @@ class ForbiddenPublicImage(Forbidden):
     message = _("You are not authorized to complete this action.")
 
 
+class ProtectedImageDelete(Forbidden):
+    message = _("Image %(image_id)s is protected and cannot be deleted.")
+
+
 #NOTE(bcwaldon): here for backwards-compatability, need to deprecate.
 class NotAuthorized(Forbidden):
     message = _("You are not authorized to complete this action.")
@@ -125,6 +140,14 @@ class InvalidFilterRangeValue(Invalid):
     message = _("Unable to filter using the specified range.")
 
 
+class ReadonlyProperty(Forbidden):
+    message = _("Attribute '%(property)s' is read-only.")
+
+
+class ReservedProperty(Forbidden):
+    message = _("Attribute '%(property)s' is reserved.")
+
+
 class AuthorizationRedirect(GlanceException):
     message = _("Redirecting to %(uri)s for authorization.")
 
@@ -189,21 +212,29 @@ class BadRegistryConnectionConfiguration(GlanceException):
 
 class BadStoreConfiguration(GlanceException):
     message = _("Store %(store_name)s could not be configured correctly. "
-               "Reason: %(reason)s")
+                "Reason: %(reason)s")
 
 
 class BadDriverConfiguration(GlanceException):
     message = _("Driver %(driver_name)s could not be configured correctly. "
-               "Reason: %(reason)s")
+                "Reason: %(reason)s")
 
 
 class StoreDeleteNotSupported(GlanceException):
     message = _("Deleting images from this store is not supported.")
 
 
+class StoreGetNotSupported(GlanceException):
+    message = _("Getting images from this store is not supported.")
+
+
+class StoreAddNotSupported(GlanceException):
+    message = _("Adding images to this store is not supported.")
+
+
 class StoreAddDisabled(GlanceException):
     message = _("Configuration for store failed. Adding images to this "
-               "store is disabled.")
+                "store is disabled.")
 
 
 class InvalidNotifierStrategy(GlanceException):
@@ -252,3 +283,7 @@ class InUseByStore(GlanceException):
 
 class ImageSizeLimitExceeded(GlanceException):
     message = _("The provided image is too large.")
+
+
+class RPCError(GlanceException):
+    message = _("%(cls)s exception was raised in the last rpc call: %(val)s")
