@@ -26,18 +26,22 @@ try:
     from eventlet import sleep
 except ImportError:
     from time import sleep
+from eventlet.green import socket
 
 import functools
 import os
 import platform
 import subprocess
 import sys
+import uuid
 
+from OpenSSL import crypto
 from oslo.config import cfg
 from webob import exc
 
 from glance.common import exception
 import glance.openstack.common.log as logging
+from glance.openstack.common import strutils
 
 CONF = cfg.CONF
 
@@ -45,6 +49,20 @@ LOG = logging.getLogger(__name__)
 
 FEATURE_BLACKLIST = ['content-length', 'content-type', 'x-image-meta-size']
 
+# Whitelist of v1 API headers of form x-image-meta-xxx
+IMAGE_META_HEADERS = ['x-image-meta-location', 'x-image-meta-size',
+                      'x-image-meta-is_public', 'x-image-meta-disk_format',
+                      'x-image-meta-container_format', 'x-image-meta-name',
+                      'x-image-meta-status', 'x-image-meta-copy_from',
+                      'x-image-meta-uri', 'x-image-meta-checksum',
+                      'x-image-meta-created_at', 'x-image-meta-updated_at',
+                      'x-image-meta-deleted_at', 'x-image-meta-min_ram',
+                      'x-image-meta-min_disk', 'x-image-meta-owner',
+                      'x-image-meta-store', 'x-image-meta-id',
+                      'x-image-meta-protected', 'x-image-meta-deleted']
+
+GLANCE_TEST_SOCKET_FD_STR = 'GLANCE_TEST_SOCKET_FD'
+
 
 def chunkreadable(iter, chunk_size=65536):
     """
@@ -83,7 +101,7 @@ def cooperative_iter(iter):
         for chunk in iter:
             sleep(0)
             yield chunk
-    except Exception, err:
+    except Exception as err:
         msg = _("Error: cooperative_iter exception %s") % err
         LOG.error(msg)
         raise
@@ -231,6 +249,9 @@ def get_image_meta_from_headers(response):
             properties[field_name] = value or None
         elif key.startswith('x-image-meta-'):
             field_name = key[len('x-image-meta-'):].replace('-', '_')
+            if 'x-image-meta-' + field_name not in IMAGE_META_HEADERS:
+                msg = _("Bad header: %(header_name)s") % {'header_name': key}
+                raise exc.HTTPBadRequest(msg, content_type="text/plain")
             result[field_name] = value or None
     result['properties'] = properties
     if 'size' in result:
@@ -240,26 +261,14 @@ def get_image_meta_from_headers(response):
             raise exception.Invalid
     for key in ('is_public', 'deleted', 'protected'):
         if key in result:
-            result[key] = bool_from_string(result[key])
+            result[key] = strutils.bool_from_string(result[key])
     return result
 
 
-def bool_from_string(subject):
-    """Interpret a string as a boolean-like value."""
-    if isinstance(subject, bool):
-        return subject
-    elif isinstance(subject, int):
-        return subject == 1
-    if hasattr(subject, 'startswith'):  # str or unicode...
-        if subject.strip().lower() in ('true', 'on', '1', 'yes', 'y'):
-            return True
-    return False
-
-
 def safe_mkdirs(path):
     try:
         os.makedirs(path)
-    except OSError, e:
+    except OSError as e:
         if e.errno != errno.EEXIST:
             raise
 
@@ -267,7 +276,7 @@ def safe_mkdirs(path):
 def safe_remove(path):
     try:
         os.remove(path)
-    except OSError, e:
+    except OSError as e:
         if e.errno != errno.ENOENT:
             raise
 
@@ -415,23 +424,25 @@ def mutating(func):
 
 
 def setup_remote_pydev_debug(host, port):
+    error_msg = ('Error setting up the debug environment.  Verify that the'
+                 ' option pydev_worker_debug_port is pointing to a valid '
+                 'hostname or IP on which a pydev server is listening on'
+                 ' the port indicated by pydev_worker_debug_port.')
 
-        error_msg = ('Error setting up the debug environment.  Verify that the'
-                     ' option pydev_worker_debug_port is pointing to a valid '
-                     'hostname or IP on which a pydev server is listening on'
-                     ' the port indicated by pydev_worker_debug_port.')
-
+    try:
         try:
             from pydev import pydevd
+        except ImportError:
+            import pydevd
 
-            pydevd.settrace(host,
-                            port=port,
-                            stdoutToServer=True,
-                            stderrToServer=True)
-            return True
-        except:
-            LOG.exception(error_msg)
-            raise
+        pydevd.settrace(host,
+                        port=port,
+                        stdoutToServer=True,
+                        stderrToServer=True)
+        return True
+    except:
+        LOG.exception(error_msg)
+        raise
 
 
 class LazyPluggable(object):
@@ -467,3 +478,49 @@ class LazyPluggable(object):
     def __getattr__(self, key):
         backend = self.__get_backend()
         return getattr(backend, key)
+
+
+def validate_key_cert(key_file, cert_file):
+    try:
+        error_key_name = "private key"
+        error_filename = key_file
+        key_str = open(key_file, "r").read()
+        key = crypto.load_privatekey(crypto.FILETYPE_PEM, key_str)
+
+        error_key_name = "certficate"
+        error_filename = cert_file
+        cert_str = open(cert_file, "r").read()
+        cert = crypto.load_certificate(crypto.FILETYPE_PEM, cert_str)
+    except IOError, ioe:
+        raise RuntimeError(_("There is a problem with your %s "
+                             "%s.  Please verify it.  Error: %s")
+                           % (error_key_name, error_filename, ioe))
+    except crypto.Error, ce:
+        raise RuntimeError(_("There is a problem with your %s "
+                             "%s.  Please verify it. OpenSSL error: %s")
+                           % (error_key_name, error_filename, ce))
+
+    try:
+        data = str(uuid.uuid4())
+        digest = "sha1"
+
+        out = crypto.sign(key, data, digest)
+        crypto.verify(cert, out, data, digest)
+    except crypto.Error, ce:
+        raise RuntimeError(_("There is a problem with your key pair.  "
+                             "Please verify that cert %s and key %s "
+                             "belong together.  OpenSSL error %s")
+                           % (cert_file, key_file, ce))
+
+
+def get_test_suite_socket():
+    global GLANCE_TEST_SOCKET_FD_STR
+    if GLANCE_TEST_SOCKET_FD_STR in os.environ:
+        fd = int(os.environ[GLANCE_TEST_SOCKET_FD_STR])
+        sock = socket.fromfd(fd, socket.AF_INET, socket.SOCK_STREAM)
+        sock = socket.SocketType(_sock=sock)
+        sock.listen(CONF.backlog)
+        del os.environ[GLANCE_TEST_SOCKET_FD_STR]
+        os.close(fd)
+        return sock
+    return None
