@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -19,15 +21,20 @@
 
 """Database setup and migration commands."""
 
-from oslo_config import cfg
-from oslo_db import options as db_options
+from glance.common import utils
 
 
-db_options.set_defaults(cfg.CONF)
+IMPL = utils.LazyPluggable('db_backend',
+                           sqlalchemy='glance.db.sqlalchemy.migration')
+
+INIT_VERSION = 000
 
 
-# Migration-related constants
-EXPAND_BRANCH = 'expand'
-CONTRACT_BRANCH = 'contract'
-CURRENT_RELEASE = '2025_2'
-ALEMBIC_INIT_VERSION = 'liberty'
+def db_sync(version=None):
+    """Migrate the database to `version` or the most recent version."""
+    return IMPL.db_sync(version=version)
+
+
+def db_version():
+    """Display the current database version."""
+    return IMPL.db_version()
