@@ -28,6 +28,7 @@ DATA = {
     'images': {},
     'members': {},
     'tags': {},
+    'locations': [],
 }
 
 
@@ -52,21 +53,46 @@ def reset():
         'images': {},
         'members': [],
         'tags': {},
+        'locations': [],
     }
 
 
 def setup_db_env(*args, **kwargs):
+    """
+    Setup global environment configuration variables.
+
+    We have no connection-oriented environment variables, so this is a NOOP.
+    """
     pass
 
 
-def configure_db(*args, **kwargs):
+def clear_db_env(*args, **kwargs):
+    """
+    Setup global environment configuration variables.
+
+    We have no connection-oriented environment variables, so this is a NOOP.
+    """
     pass
 
 
-def get_session():
+def _get_session():
     return DATA
 
 
+def _image_locations_format(image_id, value, meta_data):
+    dt = timeutils.utcnow()
+    return {
+        'id': uuidutils.generate_uuid(),
+        'image_id': image_id,
+        'created_at': dt,
+        'updated_at': dt,
+        'deleted_at': None,
+        'deleted': False,
+        'url': value,
+        'metadata': meta_data,
+    }
+
+
 def _image_property_format(image_id, name, value):
     return {
         'image_id': image_id,
@@ -113,10 +139,20 @@ def _image_format(image_id, **values):
         'deleted': False,
     }
 
+    locations = values.pop('locations', None)
+    if locations is not None:
+        locations = [
+            _image_locations_format(image_id, location['url'],
+                                    location['metadata'])
+            for location in locations
+        ]
+        image['locations'] = locations
+
     #NOTE(bcwaldon): store properties as a list to match sqlalchemy driver
     properties = values.pop('properties', {})
     properties = [{'name': k,
                    'value': v,
+                   'image_id': image_id,
                    'deleted': False} for k, v in properties.items()]
     image['properties'] = properties
 
@@ -124,15 +160,14 @@ def _image_format(image_id, **values):
     return image
 
 
-def _filter_images(images, filters, context, status='accepted'):
+def _filter_images(images, filters, context,
+                   status='accepted', is_public=None,
+                   admin_as_user=False):
     filtered_images = []
     if 'properties' in filters:
         prop_filter = filters.pop('properties')
         filters.update(prop_filter)
 
-    if 'is_public' in filters and filters['is_public'] is None:
-        filters.pop('is_public')
-
     if status == 'all':
         status = None
 
@@ -143,23 +178,29 @@ def _filter_images(images, filters, context, status='accepted'):
                                    member=context.owner, status=status)
         is_member = len(member) > 0
         has_ownership = context.owner and image['owner'] == context.owner
-        can_see = (image['is_public'] or has_ownership or context.is_admin or
-                   is_member)
+        can_see = (image['is_public'] or has_ownership or is_member or
+                   (context.is_admin and not admin_as_user))
         if not can_see:
             continue
 
         if visibility:
             if visibility == 'public':
-                filters['is_public'] = True
                 if not image['is_public']:
                     continue
             elif visibility == 'private':
-                filters['is_public'] = False
-                if not (has_ownership or context.is_admin):
+                if image['is_public']:
+                    continue
+                if not (has_ownership or (context.is_admin
+                        and not admin_as_user)):
                     continue
             elif visibility == 'shared':
                 if not is_member:
                     continue
+
+        if is_public is not None:
+            if not image['is_public'] == is_public:
+                continue
+
         add = True
         for k, value in filters.iteritems():
             key = k
@@ -175,8 +216,15 @@ def _filter_images(images, filters, context, status='accepted'):
                 add = image.get(key) >= value
             elif k.endswith('_max'):
                 add = image.get(key) <= value
-            elif image.get(k) is not None:
+            elif k != 'is_public' and image.get(k) is not None:
                 add = image.get(key) == value
+            elif k == 'tags':
+                filter_tags = value
+                image_tags = image_tag_get_all(context, image['id'])
+                for tag in filter_tags:
+                    if tag not in image_tags:
+                        add = False
+                        break
             else:
                 properties = {}
                 for p in image['properties']:
@@ -217,9 +265,10 @@ def _do_pagination(context, images, marker, limit, show_deleted,
 
 def _sort_images(images, sort_key, sort_dir):
     reverse = False
-    if images and not images[0].get(sort_key):
+    if images and not (sort_key in images[0]):
         raise exception.InvalidSortKey()
-    keyfn = lambda x: (x[sort_key], x['created_at'], x['id'])
+    keyfn = lambda x: (x[sort_key] if x[sort_key] is not None else '',
+                       x['created_at'], x['id'])
     reverse = sort_dir == 'desc'
     images.sort(key=keyfn, reverse=reverse)
 
@@ -229,6 +278,8 @@ def _sort_images(images, sort_key, sort_dir):
 def _image_get(context, image_id, force_show_deleted=False, status=None):
     try:
         image = DATA['images'][image_id]
+        image['locations'] = _image_location_get_all(image_id)
+
     except KeyError:
         LOG.info(_('Could not find image %s') % image_id)
         raise exception.NotFound()
@@ -247,19 +298,27 @@ def _image_get(context, image_id, force_show_deleted=False, status=None):
 @log_call
 def image_get(context, image_id, session=None, force_show_deleted=False):
     image = _image_get(context, image_id, force_show_deleted)
+    image = _normalize_locations(image)
     return copy.deepcopy(image)
 
 
 @log_call
 def image_get_all(context, filters=None, marker=None, limit=None,
                   sort_key='created_at', sort_dir='desc',
-                  member_status='accepted'):
+                  member_status='accepted', is_public=None,
+                  admin_as_user=False):
     filters = filters or {}
     images = DATA['images'].values()
-    images = _filter_images(images, filters, context, member_status)
+    images = _filter_images(images, filters, context, member_status,
+                            is_public, admin_as_user)
     images = _sort_images(images, sort_key, sort_dir)
     images = _do_pagination(context, images, marker, limit,
                             filters.get('deleted'))
+
+    for image in images:
+        image['locations'] = _image_location_get_all(image['id'])
+        _normalize_locations(image)
+
     return images
 
 
@@ -274,11 +333,10 @@ def image_property_create(context, values):
 
 
 @log_call
-def image_property_delete(context, prop_ref, session=None):
-    image_id = prop_ref['image_id']
+def image_property_delete(context, prop_ref, image_ref, session=None):
     prop = None
-    for p in DATA['images'][image_id]['properties']:
-        if p['name'] == prop_ref['name']:
+    for p in DATA['images'][image_ref]['properties']:
+        if p['name'] == prop_ref:
             prop = p
     if not prop:
         raise exception.NotFound()
@@ -346,6 +404,42 @@ def image_member_delete(context, member_id):
         raise exception.NotFound()
 
 
+def _image_locations_set(image_id, locations):
+    global DATA
+    image = DATA['images'][image_id]
+    for location in image['locations']:
+        location['deleted'] = True
+        location['deleted_at'] = timeutils.utcnow()
+
+    for i, location in enumerate(DATA['locations']):
+        if image_id == location['image_id'] and location['deleted'] is False:
+            del DATA['locations'][i]
+
+    for location in locations:
+        location_ref = _image_locations_format(image_id, value=location['url'],
+                                               meta_data=location['metadata'])
+        DATA['locations'].append(location_ref)
+
+        image['locations'].append(location_ref)
+
+
+def _normalize_locations(image):
+    undeleted_locations = filter(lambda x: not x['deleted'],
+                                 image['locations'])
+    image['locations'] = [{'url': loc['url'],
+                           'metadata': loc['metadata']}
+                          for loc in undeleted_locations]
+    return image
+
+
+def _image_location_get_all(image_id):
+    location_data = []
+    for location in DATA['locations']:
+        if image_id == location['image_id']:
+            location_data.append(location)
+    return location_data
+
+
 @log_call
 def image_create(context, image_values):
     global DATA
@@ -363,13 +457,21 @@ def image_create(context, image_values):
                         'created_at', 'updated_at', 'deleted_at', 'deleted',
                         'properties', 'tags'])
 
-    if set(image_values.keys()) - allowed_keys:
-        raise exception.Invalid()
+    incorrect_keys = set(image_values.keys()) - allowed_keys
+    if incorrect_keys:
+        raise exception.Invalid(
+            'The keys %s are not valid' % str(incorrect_keys))
 
     image = _image_format(image_id, **image_values)
     DATA['images'][image_id] = image
+
+    location_data = image_values.get('locations', None)
+    if location_data is not None:
+        _image_locations_set(image_id, location_data)
+
     DATA['tags'][image_id] = image.pop('tags', [])
-    return image
+
+    return _normalize_locations(copy.deepcopy(image))
 
 
 @log_call
@@ -380,6 +482,10 @@ def image_update(context, image_id, image_values, purge_props=False):
     except KeyError:
         raise exception.NotFound(image_id=image_id)
 
+    location_data = image_values.pop('locations', None)
+    if location_data is not None:
+        _image_locations_set(image_id, location_data)
+
     # replace values for properties that already exist
     new_properties = image_values.pop('properties', {})
     for prop in image['properties']:
@@ -390,13 +496,14 @@ def image_update(context, image_id, image_values, purge_props=False):
             prop['deleted'] = True
 
     # add in any completly new properties
-    image['properties'].extend([{'name': k, 'value': v, 'deleted': False}
+    image['properties'].extend([{'name': k, 'value': v,
+                                 'image_id': image_id, 'deleted': False}
                                 for k, v in new_properties.items()])
 
     image['updated_at'] = timeutils.utcnow()
     image.update(image_values)
     DATA['images'][image_id] = image
-    return image
+    return _normalize_locations(image)
 
 
 @log_call
@@ -405,6 +512,28 @@ def image_destroy(context, image_id):
     try:
         DATA['images'][image_id]['deleted'] = True
         DATA['images'][image_id]['deleted_at'] = timeutils.utcnow()
+
+        # NOTE(flaper87): Move the image to one of the deleted statuses
+        # if it hasn't been done yet.
+        if (DATA['images'][image_id]['status'] not in
+                ['deleted', 'pending_delete']):
+            DATA['images'][image_id]['status'] = 'deleted'
+
+        _image_locations_set(image_id, [])
+
+        for prop in DATA['images'][image_id]['properties']:
+            image_property_delete(context, prop['name'], image_id)
+
+        members = image_member_find(context, image_id=image_id)
+        for member in members:
+            image_member_delete(context, member['id'])
+
+        tags = image_tag_get_all(context, image_id)
+        for tag in tags:
+            image_tag_delete(context, image_id, tag)
+
+        _normalize_locations(DATA['images'][image_id])
+
         return copy.deepcopy(DATA['images'][image_id])
     except KeyError:
         raise exception.NotFound()
@@ -412,7 +541,6 @@ def image_destroy(context, image_id):
 
 @log_call
 def image_tag_get_all(context, image_id):
-    _image_get(context, image_id)
     return DATA['tags'].get(image_id, [])
 
 
@@ -521,8 +649,22 @@ def is_image_visible(context, image, status=None):
                                     image_id=image['id'],
                                     member=context.owner,
                                     status=status)
-        if len(members) > 0:
+        if members:
             return True
 
     # Private image
     return False
+
+
+def user_get_storage_usage(context, owner_id, image_id=None, session=None):
+    images = image_get_all(context, filters={'owner': owner_id})
+    total = 0
+    for image in images:
+        if image['status'] in ['killed', 'pending_delete', 'deleted']:
+            continue
+
+        if image['id'] != image_id:
+            locations = [l for l in image['locations']
+                         if not l.get('deleted', False)]
+            total += (image['size'] * len(locations))
+    return total
