@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2013 OpenStack Foundation
 # All Rights Reserved.
 #
@@ -30,3 +32,14 @@ def upgrade(migrate_engine):
                                   schema.PickleType(),
                                   default={})
     meta_data.create(image_locations_table)
+
+
+def downgrade(migrate_engine):
+    meta = sqlalchemy.schema.MetaData()
+    meta.bind = migrate_engine
+
+    image_locations_table = sqlalchemy.Table('image_locations',
+                                             meta,
+                                             autoload=True)
+
+    image_locations_table.columns['meta_data'].drop()
