@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2013 Rackspace Hosting
 # All Rights Reserved.
 #
@@ -26,3 +28,13 @@ def upgrade(migrate_engine):
 
     index = Index(INDEX_NAME, images.c.checksum)
     index.create(migrate_engine)
+
+
+def downgrade(migrate_engine):
+    meta = MetaData()
+    meta.bind = migrate_engine
+
+    images = Table('images', meta, autoload=True)
+
+    index = Index(INDEX_NAME, images.c.checksum)
+    index.drop(migrate_engine)
