@@ -45,8 +45,6 @@ image_cache_opts = [
 CONF = cfg.CONF
 CONF.register_opts(image_cache_opts)
 
-DEFAULT_MAX_CACHE_SIZE = 10 * 1024 * 1024 * 1024  # 10 GB
-
 
 class ImageCache(object):
 
@@ -65,7 +63,7 @@ class ImageCache(object):
             self.driver_class = importutils.import_class(driver_module)
             LOG.info(_("Image cache loaded driver '%s'.") %
                      driver_name)
-        except ImportError, import_err:
+        except ImportError as import_err:
             LOG.warn(_("Image cache driver "
                        "'%(driver_name)s' failed to load. "
                        "Got error: '%(import_err)s.") % locals())
@@ -83,7 +81,7 @@ class ImageCache(object):
         try:
             self.driver = self.driver_class()
             self.driver.configure()
-        except exception.BadDriverConfiguration, config_err:
+        except exception.BadDriverConfiguration as config_err:
             driver_module = self.driver_class.__module__
             LOG.warn(_("Image cache driver "
                        "'%(driver_module)s' failed to configure. "
@@ -230,57 +228,58 @@ class ImageCache(object):
 
         LOG.debug(_("Tee'ing image '%s' into cache"), image_id)
 
-        def tee_iter(image_id):
-            try:
-                current_checksum = hashlib.md5()
+        return self.cache_tee_iter(image_id, image_iter, image_checksum)
 
-                with self.driver.open_for_write(image_id) as cache_file:
-                    for chunk in image_iter:
-                        try:
-                            cache_file.write(chunk)
-                        finally:
-                            current_checksum.update(chunk)
-                            yield chunk
-                    cache_file.flush()
+    def cache_tee_iter(self, image_id, image_iter, image_checksum):
+        try:
+            current_checksum = hashlib.md5()
 
-                    if (image_checksum and
-                            image_checksum != current_checksum.hexdigest()):
-                        msg = _("Checksum verification failed. Aborted "
-                                "caching of image '%s'." % image_id)
-                        raise exception.GlanceException(msg)
+            with self.driver.open_for_write(image_id) as cache_file:
+                for chunk in image_iter:
+                    try:
+                        cache_file.write(chunk)
+                    finally:
+                        current_checksum.update(chunk)
+                        yield chunk
+                cache_file.flush()
 
-            except exception.GlanceException as e:
-                # image_iter has given us bad, (size_checked_iter has found a
-                # bad length), or corrupt data (checksum is wrong).
-                LOG.exception(e)
-                raise
-            except Exception as e:
-                LOG.exception(_("Exception encountered while tee'ing "
-                                "image '%s' into cache: %s. Continuing "
-                                "with response.") % (image_id, e))
+                if (image_checksum and
+                        image_checksum != current_checksum.hexdigest()):
+                    msg = _("Checksum verification failed. Aborted "
+                            "caching of image '%s'." % image_id)
+                    raise exception.GlanceException(msg)
 
-            # NOTE(markwash): continue responding even if caching failed
+        except exception.GlanceException as e:
+            # image_iter has given us bad, (size_checked_iter has found a
+            # bad length), or corrupt data (checksum is wrong).
+            LOG.exception(e)
+            raise
+        except Exception as e:
+            LOG.exception(_("Exception encountered while tee'ing "
+                            "image '%s' into cache: %s. Continuing "
+                            "with response.") % (image_id, e))
+
+            # If no checksum provided continue responding even if
+            # caching failed.
             for chunk in image_iter:
                 yield chunk
 
-        return tee_iter(image_id)
-
-    def cache_image_iter(self, image_id, image_iter):
+    def cache_image_iter(self, image_id, image_iter, image_checksum=None):
         """
         Cache an image with supplied iterator.
 
         :param image_id: Image ID
         :param image_file: Iterator retrieving image chunks
+        :param image_checksum: Checksum of image
 
         :retval True if image file was cached, False otherwise
         """
         if not self.driver.is_cacheable(image_id):
             return False
 
-        with self.driver.open_for_write(image_id) as cache_file:
-            for chunk in image_iter:
-                cache_file.write(chunk)
-            cache_file.flush()
+        for chunk in self.get_caching_iter(image_id, image_checksum,
+                                           image_iter):
+            pass
         return True
 
     def cache_image_file(self, image_id, image_file):
