@@ -64,7 +64,7 @@ class SqliteConnection(sqlite3.Connection):
             while True:
                 try:
                     return call()
-                except sqlite3.OperationalError, e:
+                except sqlite3.OperationalError as e:
                     if 'locked' not in str(e):
                         raise
                 sleep(0.05)
@@ -118,7 +118,7 @@ class Driver(base.Driver):
                 );
             """)
             conn.close()
-        except sqlite3.DatabaseError, e:
+        except sqlite3.DatabaseError as e:
             msg = _("Failed to initialize the image cache database. "
                     "Got error: %s") % e
             LOG.error(msg)
@@ -382,7 +382,7 @@ class Driver(base.Driver):
         conn.execute('PRAGMA temp_store = MEMORY')
         try:
             yield conn
-        except sqlite3.DatabaseError, e:
+        except sqlite3.DatabaseError as e:
             msg = _("Error executing SQLite call. Got error: %s") % e
             LOG.error(msg)
             conn.rollback()
@@ -439,8 +439,15 @@ class Driver(base.Driver):
                            will be deleted.
         """
         for path in self.get_cache_files(self.incomplete_dir):
-            os.unlink(path)
-            LOG.info(_("Removed stalled cache file %s"), path)
+            if os.path.getmtime(path) < older_than:
+                try:
+                    os.unlink(path)
+                    LOG.info(_("Removed stalled cache file %s"), path)
+                except Exception as e:
+                    msg = (_("Failed to delete file %(path)s. "
+                             "Got error: %(e)s") %
+                           dict(path=path, e=e))
+                    LOG.warn(msg)
 
     def get_queued_images(self):
         """
