@@ -25,7 +25,7 @@ from glance.common import exception
 from glance import context
 from glance.image_cache import base
 import glance.openstack.common.log as logging
-from glance import registry
+import glance.registry.client.v1.api as registry
 import glance.store
 
 
@@ -56,7 +56,11 @@ class Prefetcher(base.CacheApp):
         location = image_meta['location']
         image_data, image_size = glance.store.get_from_backend(ctx, location)
         LOG.debug(_("Caching image '%s'"), image_id)
-        self.cache.cache_image_iter(image_id, image_data)
+        cache_tee_iter = self.cache.cache_tee_iter(image_id, image_data,
+                                                   image_meta['checksum'])
+        # Image is tee'd into cache and checksum verified
+        # as we iterate
+        list(cache_tee_iter)
         return True
 
     def run(self):
