@@ -27,6 +27,7 @@ from glance.common import utils
 from glance.common import wsgi
 import glance.db
 import glance.openstack.common.log as logging
+from glance.openstack.common import strutils
 from glance.openstack.common import timeutils
 from glance.openstack.common import uuidutils
 
@@ -51,6 +52,42 @@ SUPPORTED_SORT_DIRS = ('asc', 'desc')
 SUPPORTED_PARAMS = ('limit', 'marker', 'sort_key', 'sort_dir')
 
 
+def _normalize_image_location_for_db(image_data):
+    """
+    This function takes the legacy locations field and the newly added
+    location_data field from the image_data values dictionary which flows
+    over the wire between the registry and API servers and converts it
+    into the location_data format only which is then consumable by the
+    Image object.
+
+    :param image_data: a dict of values representing information in the image
+    :return: a new image data dict
+    """
+    if 'locations' not in image_data and 'location_data' not in image_data:
+        image_data['locations'] = None
+        return image_data
+
+    locations = image_data.pop('locations', [])
+    location_data = image_data.pop('location_data', [])
+
+    location_data_dict = {}
+    for l in locations:
+        location_data_dict[l] = {}
+    for l in location_data:
+        location_data_dict[l['url']] = l['metadata']
+
+    # NOTE(jbresnah) preserve original order.  tests assume original order,
+    # should that be defined functionality
+    ordered_keys = locations[:]
+    for ld in location_data:
+        if ld['url'] not in ordered_keys:
+            ordered_keys.append(ld['url'])
+    location_data = [{'url': l, 'metadata': location_data_dict[l]}
+                     for l in ordered_keys]
+    image_data['locations'] = location_data
+    return image_data
+
+
 class Controller(object):
 
     def __init__(self):
@@ -64,13 +101,13 @@ class Controller(object):
         # NOTE(markwash): for backwards compatibility, is_public=True for
         # admins actually means "treat me as if I'm not an admin and show me
         # all my images"
-        if context.is_admin and filters.get('is_public') is True:
-            context.is_admin = False
-            del filters['is_public']
+        if context.is_admin and params.get('is_public') is True:
+            params['admin_as_user'] = True
+            del params['is_public']
         try:
             return self.db_api.image_get_all(context, filters=filters,
                                              **params)
-        except exception.NotFound, e:
+        except (exception.NotFound, exception.Forbidden) as e:
             msg = _("Invalid marker. Image could not be found.")
             raise exc.HTTPBadRequest(explanation=msg)
 
@@ -141,6 +178,10 @@ class Controller(object):
             'marker': self._get_marker(req),
         }
 
+        if req.context.is_admin:
+            # Only admin gets to look for non-public images
+            params['is_public'] = self._get_is_public(req)
+
         for key, value in params.items():
             if value is None:
                 del params[key]
@@ -160,10 +201,6 @@ class Controller(object):
         filters = {}
         properties = {}
 
-        if req.context.is_admin:
-            # Only admin gets to look for non-public images
-            filters['is_public'] = self._get_is_public(req)
-
         for param in req.params:
             if param in SUPPORTED_FILTERS:
                 filters[param] = req.params.get(param)
@@ -196,7 +233,7 @@ class Controller(object):
         elif 'changes-since' not in filters:
             filters['deleted'] = False
 
-        if len(properties) > 0:
+        if properties:
             filters['properties'] = properties
 
         return filters
@@ -273,7 +310,7 @@ class Controller(object):
         deleted = req.params.get('deleted')
         if deleted is None:
             return None
-        return utils.bool_from_string(deleted)
+        return strutils.bool_from_string(deleted)
 
     def show(self, req, id):
         """Return data about the given image id."""
@@ -358,6 +395,7 @@ class Controller(object):
             image_data['locations'] = [image_data.pop('location')]
 
         try:
+            image_data = _normalize_image_location_for_db(image_data)
             image_data = self.db_api.image_create(req.context, image_data)
             msg = _("Successfully created image %(id)s")
             LOG.info(msg % {'id': image_id})
@@ -366,7 +404,7 @@ class Controller(object):
             msg = (_("Image with identifier %s already exists!") % image_id)
             LOG.error(msg)
             return exc.HTTPConflict(msg)
-        except exception.Invalid, e:
+        except exception.Invalid as e:
             msg = (_("Failed to add image metadata. "
                      "Got error: %(e)s") % locals())
             LOG.error(msg)
@@ -396,6 +434,7 @@ class Controller(object):
         try:
             LOG.debug(_("Updating image %(id)s with metadata: "
                         "%(image_data)r") % locals())
+            image_data = _normalize_image_location_for_db(image_data)
             if purge_props == "true":
                 updated_image = self.db_api.image_update(req.context, id,
                                                          image_data, True)
@@ -405,7 +444,7 @@ class Controller(object):
             msg = _("Updating metadata for image %(id)s")
             LOG.info(msg % {'id': id})
             return dict(image=make_image_dict(updated_image))
-        except exception.Invalid, e:
+        except exception.Invalid as e:
             msg = (_("Failed to update image metadata. "
                      "Got error: %(e)s") % locals())
             LOG.error(msg)
@@ -433,9 +472,10 @@ class Controller(object):
 def _limit_locations(image):
     locations = image.pop('locations', [])
     try:
-        image['location'] = locations[0]
+        image['location'] = locations[0]['url']
     except IndexError:
         image['location'] = None
+    image['location_data'] = locations
 
 
 def make_image_dict(image):
