@@ -44,9 +44,9 @@ class Controller(object):
         try:
             self.db_api.image_get(req.context, image_id)
         except exception.NotFound:
-            msg = _("Image %(id)s not found")
-            LOG.info(msg % {'id': image_id})
-            raise webob.exc.HTTPNotFound()
+            msg = _("Image %(id)s not found") % {'id': image_id}
+            LOG.info(msg)
+            raise webob.exc.HTTPNotFound(msg)
         except exception.Forbidden:
             # If it's private and doesn't belong to them, don't let on
             # that it exists
@@ -75,14 +75,12 @@ class Controller(object):
         self._check_can_access_image_members(req.context)
 
         # Make sure the image exists
-        session = self.db_api.get_session()
         try:
-            image = self.db_api.image_get(req.context, image_id,
-                                          session=session)
+            image = self.db_api.image_get(req.context, image_id)
         except exception.NotFound:
-            msg = _("Image %(id)s not found")
-            LOG.info(msg % {'id': image_id})
-            raise webob.exc.HTTPNotFound()
+            msg = _("Image %(id)s not found") % {'id': image_id}
+            LOG.info(msg)
+            raise webob.exc.HTTPNotFound(msg)
         except exception.Forbidden:
             # If it's private and doesn't belong to them, don't let on
             # that it exists
@@ -100,7 +98,7 @@ class Controller(object):
         # Get the membership list
         try:
             memb_list = body['memberships']
-        except Exception, e:
+        except Exception as e:
             # Malformed entity...
             msg = _("Invalid membership association specified for "
                     "image %(id)s")
@@ -116,7 +114,7 @@ class Controller(object):
                 datum = dict(image_id=image['id'],
                              member=memb['member_id'],
                              can_share=None)
-            except Exception, e:
+            except Exception as e:
                 # Malformed entity...
                 msg = _("Invalid membership association specified for "
                         "image %(id)s")
@@ -166,7 +164,7 @@ class Controller(object):
 
         # Now add the non-existent ones
         for memb in add:
-            self.db_api.image_member_create(req.context, memb, session=session)
+            self.db_api.image_member_create(req.context, memb)
 
         # Make an appropriate result
         msg = _("Successfully updated memberships for image %(id)s")
@@ -193,9 +191,9 @@ class Controller(object):
         try:
             image = self.db_api.image_get(req.context, image_id)
         except exception.NotFound:
-            msg = _("Image %(id)s not found")
-            LOG.info(msg % {'id': image_id})
-            raise webob.exc.HTTPNotFound()
+            msg = _("Image %(id)s not found") % {'id': image_id}
+            LOG.info(msg)
+            raise webob.exc.HTTPNotFound(msg)
         except exception.Forbidden:
             # If it's private and doesn't belong to them, don't let on
             # that it exists
@@ -215,7 +213,7 @@ class Controller(object):
         if body:
             try:
                 can_share = bool(body['member']['can_share'])
-            except Exception, e:
+            except Exception as e:
                 # Malformed entity...
                 msg = _("Invalid membership association specified for "
                         "image %(id)s")
@@ -224,7 +222,6 @@ class Controller(object):
                 raise webob.exc.HTTPBadRequest(explanation=msg)
 
         # Look up an existing membership...
-        session = self.db_api.get_session()
         members = self.db_api.image_member_find(req.context,
                                                 image_id=image_id,
                                                 member=id)
@@ -237,8 +234,7 @@ class Controller(object):
         else:
             values = dict(image_id=image['id'], member=id,
                           can_share=bool(can_share))
-            self.db_api.image_member_create(req.context, values,
-                                            session=session)
+            self.db_api.image_member_create(req.context, values)
 
         msg = _("Successfully updated a membership for image %(id)s")
         LOG.info(msg % {'id': image_id})
@@ -255,9 +251,9 @@ class Controller(object):
         try:
             image = self.db_api.image_get(req.context, image_id)
         except exception.NotFound:
-            msg = _("Image %(id)s not found")
-            LOG.info(msg % {'id': image_id})
-            raise webob.exc.HTTPNotFound()
+            msg = _("Image %(id)s not found") % {'id': image_id}
+            LOG.info(msg)
+            raise webob.exc.HTTPNotFound(msg)
         except exception.Forbidden:
             # If it's private and doesn't belong to them, don't let on
             # that it exists
@@ -295,7 +291,7 @@ class Controller(object):
         """
         try:
             members = self.db_api.image_member_find(req.context, member=id)
-        except exception.NotFound, e:
+        except exception.NotFound as e:
             msg = _("Member %(id)s not found")
             LOG.info(msg % {'id': id})
             msg = _("Membership could not be found.")
