@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2013 Red Hat, Inc.
 # All Rights Reserved.
 #
@@ -13,23 +15,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import routes
+from webob import exc
+
 from glance.common import wsgi
 from glance.registry.api.v2 import rpc
 
 
-def init(mapper):
-    rpc_resource = rpc.create_resource()
-    mapper.connect("/rpc", controller=rpc_resource,
-                   conditions=dict(method=["POST"]),
-                   action="__call__")
-
-
 class API(wsgi.Router):
     """WSGI entry point for all Registry requests."""
 
     def __init__(self, mapper):
-        mapper = mapper or wsgi.APIMapper()
-
-        init(mapper)
+        mapper = mapper or routes.Mapper()
 
+        rpc_resource = rpc.create_resource()
+        mapper.connect("/rpc", controller=rpc_resource,
+                       conditions=dict(method=["POST"]),
+                       action="__call__")
         super(API, self).__init__(mapper)
