@@ -1,4 +1,6 @@
-# Copyright 2010-2011 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2010-2011 OpenStack, LLC
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,14 +19,13 @@
 Registry's Client API
 """
 
+import json
 import os
 
-from oslo_config import cfg
-from oslo_log import log as logging
-from oslo_serialization import jsonutils
+from oslo.config import cfg
 
 from glance.common import exception
-from glance.i18n import _
+import glance.openstack.common.log as logging
 from glance.registry.client.v1 import client
 
 LOG = logging.getLogger(__name__)
@@ -34,7 +35,7 @@ registry_client_ctx_opts = [
                 help=_("Whether to pass through headers containing user "
                        "and tenant information when making requests to "
                        "the registry. This allows the registry to use the "
-                       "context middleware without keystonemiddleware's "
+                       "context middleware without the keystoneclients' "
                        "auth_token middleware, removing calls to the keystone "
                        "auth service. It is recommended that when using this "
                        "option, secure communication between glance api and "
@@ -113,7 +114,7 @@ def configure_registry_admin_creds():
         'password': CONF.admin_password,
         'username': CONF.admin_user,
         'tenant': CONF.admin_tenant_name,
-        'auth_url': os.getenv('OS_AUTH_URL') or CONF.auth_url,
+        'auth_url': CONF.auth_url,
         'strategy': strategy,
         'region': CONF.auth_region,
     }
@@ -124,22 +125,19 @@ def get_registry_client(cxt):
     global _METADATA_ENCRYPTION_KEY
     kwargs = _CLIENT_KWARGS.copy()
     if CONF.use_user_token:
-        kwargs['auth_token'] = cxt.auth_token
+        kwargs['auth_tok'] = cxt.auth_tok
     if _CLIENT_CREDS:
         kwargs['creds'] = _CLIENT_CREDS
 
     if CONF.send_identity_headers:
         identity_headers = {
-            'X-User-Id': cxt.user or '',
-            'X-Tenant-Id': cxt.tenant or '',
+            'X-User-Id': cxt.user,
+            'X-Tenant-Id': cxt.tenant,
             'X-Roles': ','.join(cxt.roles),
             'X-Identity-Status': 'Confirmed',
-            'X-Service-Catalog': jsonutils.dumps(cxt.service_catalog),
+            'X-Service-Catalog': json.dumps(cxt.service_catalog),
         }
         kwargs['identity_headers'] = identity_headers
-
-    kwargs['request_id'] = cxt.request_id
-
     return client.RegistryClient(_CLIENT_HOST, _CLIENT_PORT,
                                  _METADATA_ENCRYPTION_KEY, **kwargs)
 
@@ -160,21 +158,20 @@ def get_image_metadata(context, image_id):
 
 
 def add_image_metadata(context, image_meta):
-    LOG.debug("Adding image metadata...")
+    LOG.debug(_("Adding image metadata..."))
     c = get_registry_client(context)
     return c.add_image(image_meta)
 
 
 def update_image_metadata(context, image_id, image_meta,
-                          purge_props=False, from_state=None):
-    LOG.debug("Updating image metadata for image %s...", image_id)
+                          purge_props=False):
+    LOG.debug(_("Updating image metadata for image %s..."), image_id)
     c = get_registry_client(context)
-    return c.update_image(image_id, image_meta, purge_props=purge_props,
-                          from_state=from_state)
+    return c.update_image(image_id, image_meta, purge_props)
 
 
 def delete_image_metadata(context, image_id):
-    LOG.debug("Deleting image metadata for image %s...", image_id)
+    LOG.debug(_("Deleting image metadata for image %s..."), image_id)
     c = get_registry_client(context)
     return c.delete_image(image_id)
 
