@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2013 Red Hat, Inc
 # All Rights Reserved.
 #
@@ -19,11 +21,10 @@ Registry's Client V2
 
 import os
 
-from oslo_config import cfg
-from oslo_log import log as logging
+from oslo.config import cfg
 
 from glance.common import exception
-from glance.i18n import _
+import glance.openstack.common.log as logging
 from glance.registry.client.v2 import client
 
 LOG = logging.getLogger(__name__)
@@ -32,6 +33,7 @@ CONF = cfg.CONF
 _registry_client = 'glance.registry.client'
 CONF.import_opt('registry_client_protocol', _registry_client)
 CONF.import_opt('registry_client_key_file', _registry_client)
+CONF.import_opt('registry_client_cert_file', _registry_client)
 CONF.import_opt('registry_client_ca_file', _registry_client)
 CONF.import_opt('registry_client_insecure', _registry_client)
 CONF.import_opt('registry_client_timeout', _registry_client)
@@ -93,7 +95,7 @@ def configure_registry_admin_creds():
         'password': CONF.admin_password,
         'username': CONF.admin_user,
         'tenant': CONF.admin_tenant_name,
-        'auth_url': os.getenv('OS_AUTH_URL') or CONF.auth_url,
+        'auth_url': CONF.auth_url,
         'strategy': strategy,
         'region': CONF.auth_region,
     }
@@ -103,7 +105,7 @@ def get_registry_client(cxt):
     global _CLIENT_CREDS, _CLIENT_KWARGS, _CLIENT_HOST, _CLIENT_PORT
     kwargs = _CLIENT_KWARGS.copy()
     if CONF.use_user_token:
-        kwargs['auth_token'] = cxt.auth_token
+        kwargs['auth_tok'] = cxt.auth_tok
     if _CLIENT_CREDS:
         kwargs['creds'] = _CLIENT_CREDS
     return client.RegistryClient(_CLIENT_HOST, _CLIENT_PORT, **kwargs)
