@@ -19,17 +19,31 @@
 """Base class for all storage backends"""
 
 from glance.common import exception
+from glance.common import utils
 from glance.openstack.common import importutils
 import glance.openstack.common.log as logging
+from glance.openstack.common import strutils
 
 LOG = logging.getLogger(__name__)
 
 
+def _exception_to_unicode(exc):
+    try:
+        return unicode(exc)
+    except UnicodeError:
+        try:
+            return strutils.safe_decode(str(exc), errors='ignore')
+        except UnicodeError:
+            msg = (_("Caught '%(exception)s' exception.") %
+                   {"exception": exc.__class__.__name__})
+            return strutils.safe_decode(msg, errors='ignore')
+
+
 class Store(object):
 
     CHUNKSIZE = (16 * 1024 * 1024)  # 16M
 
-    def __init__(self, context=None):
+    def __init__(self, context=None, location=None):
         """
         Initialize the Store
         """
@@ -39,11 +53,11 @@ class Store(object):
 
         try:
             self.configure_add()
-        except exception.BadStoreConfiguration:
-            msg = _("Failed to configure store correctly. "
-                    "Disabling add method.")
-            LOG.error(msg)
+        except exception.BadStoreConfiguration as e:
             self.add = self.add_disabled
+            msg = (_(u"Failed to configure store correctly: %s "
+                     "Disabling add method.") % _exception_to_unicode(e))
+            LOG.warn(msg)
 
     def configure(self):
         """
@@ -113,14 +127,15 @@ class Store(object):
     def add(self, image_id, image_file, image_size):
         """
         Stores an image file with supplied identifier to the backend
-        storage system and returns an `glance.store.ImageAddResult` object
-        containing information about the stored image.
+        storage system and returns a tuple containing information
+        about the stored image.
 
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
 
-        :retval `glance.store.ImageAddResult` object
+        :retval tuple of URL in backing store, bytes written, checksum
+               and a dictionary with storage system specific information
         :raises `glance.common.exception.Duplicate` if the image already
                 existed
         """
