@@ -66,10 +66,11 @@ def get_location_from_uri(uri):
         s3://accesskey:secretkey@s3.amazonaws.com/bucket/key-id
         s3+https://accesskey:secretkey@s3.amazonaws.com/bucket/key-id
         file:///var/lib/glance/images/1
+        cinder://volume-id
     """
     pieces = urlparse.urlparse(uri)
     if pieces.scheme not in SCHEME_TO_CLS_MAP.keys():
-        raise exception.UnknownScheme(pieces.scheme)
+        raise exception.UnknownScheme(scheme=pieces.scheme)
     scheme_info = SCHEME_TO_CLS_MAP[pieces.scheme]
     return Location(pieces.scheme, uri=uri,
                     store_location_class=scheme_info['location_class'])
