@@ -13,28 +13,28 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_config import cfg
-from oslo_db import options
-
-import glance.db
+from glance.api import CONF
+import glance.db.registry.api
+from glance.db.sqlalchemy import api
+from glance.db.sqlalchemy import models as db_models
 import glance.tests.functional.db as db_tests
+from glance.tests import functional
 from glance.tests.functional.db import base
-from glance.tests.functional.db import base_metadef
-
-CONF = cfg.CONF
 
 
 def get_db(config):
-    options.set_defaults(CONF, connection='sqlite://')
-    config(data_api='glance.db.registry.api')
-    return glance.db.get_api()
+    config(sql_connection='sqlite://', verbose=False, debug=False)
+    CONF.set_override('data_api', 'glance.db.registry.api')
+    db_api = glance.db.get_api()
+    db_api.setup_db_env()
+    return db_api
 
 
 def reset_db(db_api):
     pass
 
 
-class FunctionalInitWrapper(base.FunctionalInitWrapper):
+class FunctionalInitWrapper(functional.FunctionalTest):
 
     def setUp(self):
         # NOTE(flaper87): We need to start the
@@ -85,28 +85,3 @@ class TestRegistryQuota(base.DriverQuotaTests, FunctionalInitWrapper):
     def tearDown(self):
         self.registry_server.stop()
         super(TestRegistryQuota, self).tearDown()
-
-
-class TestRegistryMetadefDriver(base_metadef.TestMetadefDriver,
-                                base_metadef.MetadefDriverTests,
-                                FunctionalInitWrapper):
-
-    def setUp(self):
-        db_tests.load(get_db, reset_db)
-        super(TestRegistryMetadefDriver, self).setUp()
-        self.addCleanup(db_tests.reset)
-
-    def tearDown(self):
-        self.registry_server.stop()
-        super(TestRegistryMetadefDriver, self).tearDown()
-
-
-class TestTasksDriver(base.TaskTests, FunctionalInitWrapper):
-    def setUp(self):
-        db_tests.load(get_db, reset_db)
-        super(TestTasksDriver, self).setUp()
-        self.addCleanup(db_tests.reset)
-
-    def tearDown(self):
-        self.registry_server.stop()
-        super(TestTasksDriver, self).tearDown()
