@@ -1,4 +1,6 @@
-# Copyright 2011 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2011 OpenStack, LLC
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,21 +19,15 @@
 
 import datetime
 import hashlib
+import httplib2
+import json
 import os
 import sys
 
-import httplib2
-from oslo_serialization import jsonutils
-from oslo_utils import units
-from six.moves import http_client
-# NOTE(jokke): simplified transition to py3, behaves like py2 xrange
-from six.moves import range
-
 from glance.tests import functional
-from glance.tests.utils import execute
-from glance.tests.utils import minimal_headers
+from glance.tests.utils import execute, minimal_headers
 
-FIVE_KB = 5 * units.Ki
+FIVE_KB = 5 * 1024
 
 
 class TestBinGlanceCacheManage(functional.FunctionalTest):
@@ -56,19 +52,19 @@ class TestBinGlanceCacheManage(functional.FunctionalTest):
         Adds an image with supplied name and returns the newly-created
         image identifier.
         """
-        image_data = b"*" * FIVE_KB
+        image_data = "*" * FIVE_KB
         headers = minimal_headers(name)
         path = "http://%s:%d/v1/images" % ("127.0.0.1", self.api_port)
         http = httplib2.Http()
         response, content = http.request(path, 'POST', headers=headers,
                                          body=image_data)
-        self.assertEqual(http_client.CREATED, response.status)
-        data = jsonutils.loads(content)
-        self.assertEqual(hashlib.md5(image_data).hexdigest(),
-                         data['image']['checksum'])
-        self.assertEqual(FIVE_KB, data['image']['size'])
-        self.assertEqual(name, data['image']['name'])
-        self.assertTrue(data['image']['is_public'])
+        self.assertEqual(response.status, 201)
+        data = json.loads(content)
+        self.assertEqual(data['image']['checksum'],
+                         hashlib.md5(image_data).hexdigest())
+        self.assertEqual(data['image']['size'], FIVE_KB)
+        self.assertEqual(data['image']['name'], name)
+        self.assertEqual(data['image']['is_public'], True)
         return data['image']['id']
 
     def is_image_cached(self, image_id):
@@ -81,7 +77,6 @@ class TestBinGlanceCacheManage(functional.FunctionalTest):
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
-        out = out.decode('utf-8')
         return image_id in out
 
     def iso_date(self, image_id):
@@ -92,7 +87,6 @@ class TestBinGlanceCacheManage(functional.FunctionalTest):
         cmd = "%s --port=%d list-cached" % (exe_cmd, self.api_port)
 
         exitcode, out, err = execute(cmd)
-        out = out.decode('utf-8')
 
         return datetime.datetime.utcnow().strftime("%Y-%m-%d") in out
 
@@ -113,8 +107,8 @@ class TestBinGlanceCacheManage(functional.FunctionalTest):
         exitcode, out, err = execute(cmd, raise_error=False)
 
         self.assertEqual(1, exitcode)
-        self.assertIn(b'Cache management middleware not enabled on host',
-                      out.strip())
+        self.assertTrue('Cache management middleware not enabled on host'
+                        in out.strip())
 
         self.stop_servers()
 
@@ -134,20 +128,20 @@ class TestBinGlanceCacheManage(functional.FunctionalTest):
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
-        self.assertIn(b'No cached images', out.strip())
+        self.assertTrue('No cached images' in out.strip())
 
         ids = {}
 
         # Add a few images and cache the second one of them
         # by GETing the image...
-        for x in range(4):
+        for x in xrange(0, 4):
             ids[x] = self.add_image("Image%s" % x)
 
         path = "http://%s:%d/v1/images/%s" % ("127.0.0.1", api_port,
                                               ids[1])
         http = httplib2.Http()
         response, content = http.request(path, 'GET')
-        self.assertEqual(http_client.OK, response.status)
+        self.assertEqual(response.status, 200)
 
         self.assertTrue(self.is_image_cached(ids[1]),
                         "%s is not cached." % ids[1])
@@ -173,7 +167,7 @@ class TestBinGlanceCacheManage(functional.FunctionalTest):
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
-        self.assertIn(b'No cached images', out.strip())
+        self.assertTrue('No cached images' in out.strip())
 
         # Verify no queued images
         cmd = "%s --port=%d list-queued" % (exe_cmd, api_port)
@@ -181,18 +175,18 @@ class TestBinGlanceCacheManage(functional.FunctionalTest):
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
-        self.assertIn(b'No queued images', out.strip())
+        self.assertTrue('No queued images' in out.strip())
 
         ids = {}
 
         # Add a few images and cache the second one of them
         # by GETing the image...
-        for x in range(4):
+        for x in xrange(0, 4):
             ids[x] = self.add_image("Image%s" % x)
 
         # Queue second image and then cache it
         cmd = "%s --port=%d --force queue-image %s" % (
-            exe_cmd, api_port, ids[1])
+                exe_cmd, api_port, ids[1])
 
         exitcode, out, err = execute(cmd)
 
@@ -204,8 +198,7 @@ class TestBinGlanceCacheManage(functional.FunctionalTest):
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
-        out = out.decode('utf-8')
-        self.assertIn(ids[1], out, 'Image %s was not queued!' % ids[1])
+        self.assertTrue(ids[1] in out, 'Image %s was not queued!' % ids[1])
 
         # Cache images in the queue by running the prefetcher
         cache_config_filepath = os.path.join(self.test_dir, 'etc',
@@ -214,24 +207,19 @@ class TestBinGlanceCacheManage(functional.FunctionalTest):
             'image_cache_dir': self.api_server.image_cache_dir,
             'image_cache_driver': self.image_cache_driver,
             'registry_port': self.registry_server.bind_port,
-            'lock_path': self.test_dir,
             'log_file': os.path.join(self.test_dir, 'cache.log'),
-            'metadata_encryption_key': "012345678901234567890123456789ab",
-            'filesystem_store_datadir': self.test_dir
+            'metadata_encryption_key': "012345678901234567890123456789ab"
         }
         with open(cache_config_filepath, 'w') as cache_file:
             cache_file.write("""[DEFAULT]
 debug = True
-lock_path = %(lock_path)s
+verbose = True
 image_cache_dir = %(image_cache_dir)s
 image_cache_driver = %(image_cache_driver)s
 registry_host = 127.0.0.1
 registry_port = %(registry_port)s
 metadata_encryption_key = %(metadata_encryption_key)s
 log_file = %(log_file)s
-
-[glance_store]
-filesystem_store_datadir=%(filesystem_store_datadir)s
 """ % cache_file_options)
 
         cmd = ("%s -m glance.cmd.cache_prefetcher --config-file %s" %
@@ -240,7 +228,7 @@ filesystem_store_datadir=%(filesystem_store_datadir)s
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
-        self.assertEqual(b'', out.strip(), out)
+        self.assertEqual('', out.strip(), out)
 
         # Verify no queued images
         cmd = "%s --port=%d list-queued" % (exe_cmd, api_port)
@@ -248,7 +236,7 @@ filesystem_store_datadir=%(filesystem_store_datadir)s
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
-        self.assertIn(b'No queued images', out.strip())
+        self.assertTrue('No queued images' in out.strip())
 
         # Verify second image now cached
         cmd = "%s --port=%d list-cached" % (exe_cmd, api_port)
@@ -256,12 +244,11 @@ filesystem_store_datadir=%(filesystem_store_datadir)s
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
-        out = out.decode('utf-8')
-        self.assertIn(ids[1], out, 'Image %s was not cached!' % ids[1])
+        self.assertTrue(ids[1] in out, 'Image %s was not cached!' % ids[1])
 
         # Queue third image and then delete it from queue
         cmd = "%s --port=%d --force queue-image %s" % (
-            exe_cmd, api_port, ids[2])
+                exe_cmd, api_port, ids[2])
 
         exitcode, out, err = execute(cmd)
 
@@ -273,8 +260,7 @@ filesystem_store_datadir=%(filesystem_store_datadir)s
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
-        out = out.decode('utf-8')
-        self.assertIn(ids[2], out, 'Image %s was not queued!' % ids[2])
+        self.assertTrue(ids[2] in out, 'Image %s was not queued!' % ids[2])
 
         # Delete the image from the queue
         cmd = ("%s --port=%d --force "
@@ -290,10 +276,10 @@ filesystem_store_datadir=%(filesystem_store_datadir)s
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
-        self.assertIn(b'No queued images', out.strip())
+        self.assertTrue('No queued images' in out.strip())
 
         # Queue all images
-        for x in range(4):
+        for x in xrange(0, 4):
             cmd = ("%s --port=%d --force "
                    "queue-image %s") % (exe_cmd, api_port, ids[x])
 
@@ -307,7 +293,7 @@ filesystem_store_datadir=%(filesystem_store_datadir)s
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
-        self.assertIn(b'Found 3 queued images', out)
+        self.assertTrue('Found 3 queued images' in out)
 
         # Delete the image from the queue
         cmd = ("%s --port=%d --force "
@@ -323,36 +309,6 @@ filesystem_store_datadir=%(filesystem_store_datadir)s
         exitcode, out, err = execute(cmd)
 
         self.assertEqual(0, exitcode)
-        self.assertIn(b'No queued images', out.strip())
-
-        # verify two image id when queue-image
-        cmd = ("%s --port=%d --force "
-               "queue-image %s %s") % (exe_cmd, api_port, ids[0], ids[1])
-
-        exitcode, out, err = execute(cmd, raise_error=False)
-
-        self.assertEqual(1, exitcode)
-        self.assertIn(b'Please specify one and only ID of '
-                      b'the image you wish to ', out.strip())
-
-        # verify two image id when delete-queued-image
-        cmd = ("%s --port=%d --force delete-queued-image "
-               "%s %s") % (exe_cmd, api_port, ids[0], ids[1])
-
-        exitcode, out, err = execute(cmd, raise_error=False)
-
-        self.assertEqual(1, exitcode)
-        self.assertIn(b'Please specify one and only ID of '
-                      b'the image you wish to ', out.strip())
-
-        # verify two image id when delete-cached-image
-        cmd = ("%s --port=%d --force delete-cached-image "
-               "%s %s") % (exe_cmd, api_port, ids[0], ids[1])
-
-        exitcode, out, err = execute(cmd, raise_error=False)
-
-        self.assertEqual(1, exitcode)
-        self.assertIn(b'Please specify one and only ID of '
-                      b'the image you wish to ', out.strip())
+        self.assertTrue('No queued images' in out.strip())
 
         self.stop_servers()
