@@ -19,6 +19,7 @@
 """Functional test asserting strongly typed exceptions from glance client"""
 
 import eventlet.patcher
+import httplib2
 import webob.dec
 import webob.exc
 
@@ -62,6 +63,9 @@ class ExceptionTestApp(object):
         elif path == "/server-error":
             request.response = webob.exc.HTTPServerError()
 
+        elif path == "/server-traceback":
+            raise exception.ServerError()
+
 
 class TestClientExceptions(functional.FunctionalTest):
 
@@ -69,8 +73,9 @@ class TestClientExceptions(functional.FunctionalTest):
         super(TestClientExceptions, self).setUp()
         self.port = utils.get_unused_port()
         server = wsgi.Server()
-        conf = utils.TestConfigOpts({'bind_host': '127.0.0.1'})
-        server.start(ExceptionTestApp(), conf, self.port)
+        self.config(bind_host='127.0.0.1')
+        self.config(workers=0)
+        server.start(ExceptionTestApp(), self.port)
         self.client = client.BaseClient("127.0.0.1", self.port)
 
     def _do_test_exception(self, path, exc_type):
@@ -120,3 +125,15 @@ class TestClientExceptions(functional.FunctionalTest):
         """
         self._do_test_exception('/server-error',
                                 exception.ServerError)
+
+    def test_server_traceback(self):
+        """
+        Verify that the wsgi server does not return tracebacks to the client on
+        500 errors (bug 1192132)
+        """
+        http = httplib2.Http()
+        path = ('http://%s:%d/server-traceback' %
+                ('127.0.0.1', self.port))
+        response, content = http.request(path, 'GET')
+        self.assertTrue('ServerError' not in content)
+        self.assertEqual(response.status, 500)
