@@ -18,6 +18,7 @@
 """Functional test cases for glance-manage"""
 
 import os
+import sys
 
 from glance.common import utils
 from glance.tests import functional
@@ -43,8 +44,8 @@ class TestGlanceManage(functional.FunctionalTest):
             conf_file.write(self.connection)
             conf_file.flush()
 
-        cmd = ('bin/glance-manage --config-file %s db_sync' %
-               self.conf_filepath)
+        cmd = ('%s -m glance.cmd.manage --config-file %s db_sync' %
+               (sys.executable, self.conf_filepath))
         execute(cmd, raise_error=True)
 
     def _assert_tables(self):
@@ -68,8 +69,6 @@ class TestGlanceManage(functional.FunctionalTest):
 
         self._assert_tables()
 
-        self.stop_servers()
-
     @depends_on_exe('sqlite3')
     @skip_if_disabled
     def test_db_creation_auto_create_overridden(self):
@@ -77,5 +76,3 @@ class TestGlanceManage(functional.FunctionalTest):
         self._sync_db(False)
 
         self._assert_tables()
-
-        self.stop_servers()
