@@ -17,6 +17,7 @@
 
 import json
 import os
+import sys
 import time
 
 import httplib2
@@ -78,7 +79,7 @@ class TestScrubber(functional.FunctionalTest):
         self.assertEqual(response.status, 200)
         self.assertEqual('pending_delete', response['x-image-meta-status'])
 
-        wait_for_scrub(path)
+        self.wait_for_scrub(path)
 
         self.stop_servers()
 
@@ -120,12 +121,13 @@ class TestScrubber(functional.FunctionalTest):
         time.sleep(self.api_server.scrub_time)
 
         # scrub images and make sure they get deleted
-        cmd = ("bin/glance-scrubber --config-file %s" %
-               self.scrubber_daemon.conf_file_name)
+        exe_cmd = "%s -m glance.cmd.scrubber" % sys.executable
+        cmd = ("%s --config-file %s" %
+               (exe_cmd, self.scrubber_daemon.conf_file_name))
         exitcode, out, err = execute(cmd, raise_error=False)
         self.assertEqual(0, exitcode)
 
-        wait_for_scrub(path)
+        self.wait_for_scrub(path)
 
         self.stop_servers()
 
@@ -180,13 +182,14 @@ class TestScrubber(functional.FunctionalTest):
         time.sleep(self.api_server.scrub_time)
 
         # call the scrubber to scrub images
-        cmd = ("bin/glance-scrubber --config-file %s" %
-               self.scrubber_daemon.conf_file_name)
+        exe_cmd = "%s -m glance.cmd.scrubber" % sys.executable
+        cmd = ("%s --config-file %s" %
+               (exe_cmd, self.scrubber_daemon.conf_file_name))
         exitcode, out, err = execute(cmd, raise_error=False)
         self.assertEqual(0, exitcode)
 
         # ensure the image has been successfully deleted
-        wait_for_scrub(path)
+        self.wait_for_scrub(path)
 
         self.stop_servers()
 
@@ -239,7 +242,7 @@ class TestScrubber(functional.FunctionalTest):
         # it and checking the image_id is intact
         file_path = os.path.join(self.api_server.scrubber_datadir,
                                  str(image_id))
-        marker_uri = ''
+        marker_uri = None
         with open(file_path, 'r') as f:
             marker_uri = f.readline().strip()
         self.assertTrue(marker_uri is not None)
@@ -249,10 +252,10 @@ class TestScrubber(functional.FunctionalTest):
         loc = StoreLocation({})
         loc.parse_uri(decrypted_uri)
 
-        self.assertEqual("swift+http", loc.scheme)
+        self.assertIn(loc.scheme, ("swift+http", "swift+https"))
         self.assertEqual(image['id'], loc.obj)
 
-        wait_for_scrub(path)
+        self.wait_for_scrub(path)
 
         self.stop_servers()
 
@@ -318,33 +321,101 @@ class TestScrubber(functional.FunctionalTest):
         time.sleep(self.api_server.scrub_time)
 
         # run the scrubber app, and ensure it doesn't fall over
-        cmd = ("bin/glance-scrubber --config-file %s" %
-               self.scrubber_daemon.conf_file_name)
+        exe_cmd = "%s -m glance.cmd.scrubber" % sys.executable
+        cmd = ("%s --config-file %s" %
+               (exe_cmd, self.scrubber_daemon.conf_file_name))
         exitcode, out, err = execute(cmd, raise_error=False)
         self.assertEqual(0, exitcode)
 
-        wait_for_scrub(path)
+        self.wait_for_scrub(path)
 
         self.stop_servers()
 
+    def test_scrubber_delete_handles_exception(self):
+        """
+        Test that the scrubber handles the case where an
+        exception occurs when _delete() is called. The scrubber
+        should not write out queue files in this case.
+        """
 
-def wait_for_scrub(path):
-    """
-    NOTE(jkoelker) The build servers sometimes take longer than 15 seconds to
-    scrub. Give it up to 5 min, checking checking every 15 seconds. When/if it
-    flips to deleted, bail immediatly.
-    """
-    http = httplib2.Http()
-    wait_for = 300    # seconds
-    check_every = 15  # seconds
-    for _ in xrange(wait_for / check_every):
-        time.sleep(check_every)
+        # Start servers.
+        self.cleanup()
+        self.start_servers(delayed_delete=True, daemon=False,
+                           default_store='file')
 
+        # Check that we are using a file backend.
+        self.assertEqual(self.api_server.default_store, 'file')
+
+        # add an image
+        headers = {
+            'x-image-meta-name': 'test_image',
+            'x-image-meta-is_public': 'true',
+            'x-image-meta-disk_format': 'raw',
+            'x-image-meta-container_format': 'ovf',
+            'content-type': 'application/octet-stream',
+        }
+        path = "http://%s:%d/v1/images" % ("127.0.0.1", self.api_port)
+        http = httplib2.Http()
+        response, content = http.request(path, 'POST', body='XXX',
+                                         headers=headers)
+        self.assertEqual(response.status, 201)
+        image = json.loads(content)['image']
+        self.assertEqual('active', image['status'])
+        image_id = image['id']
+
+        # delete the image
+        path = "http://%s:%d/v1/images/%s" % ("127.0.0.1", self.api_port,
+                                              image_id)
+        http = httplib2.Http()
+        response, content = http.request(path, 'DELETE')
+        self.assertEqual(response.status, 200)
+
+        # ensure the image is marked pending delete
         response, content = http.request(path, 'HEAD')
-        if (response['x-image-meta-status'] == 'deleted' and
-                response['x-image-meta-deleted'] == 'True'):
-            break
+        self.assertEqual(response.status, 200)
+        self.assertEqual('pending_delete', response['x-image-meta-status'])
+
+        # Remove the file from the backend.
+        file_path = os.path.join(self.api_server.image_dir,
+                                 str(image_id))
+        os.remove(file_path)
+
+        # Wait for the scrub time on the image to pass
+        time.sleep(self.api_server.scrub_time)
+
+        # run the scrubber app, and ensure it doesn't fall over
+        exe_cmd = "%s -m glance.cmd.scrubber" % sys.executable
+        cmd = ("%s --config-file %s" %
+               (exe_cmd, self.scrubber_daemon.conf_file_name))
+        exitcode, out, err = execute(cmd, raise_error=False)
+        self.assertEqual(0, exitcode)
+
+        self.wait_for_scrub(path)
+
+        # Make sure there are no queue files associated with image.
+        queue_file_path = os.path.join(self.api_server.scrubber_datadir,
+                                       str(image_id))
+        self.assertFalse(os.path.exists(queue_file_path))
+
+        self.stop_servers()
+
+    def wait_for_scrub(self, path):
+        """
+        NOTE(jkoelker) The build servers sometimes take longer than 15 seconds
+        to scrub. Give it up to 5 min, checking checking every 15 seconds.
+        When/if it flips to deleted, bail immediatly.
+        """
+        http = httplib2.Http()
+        wait_for = 300    # seconds
+        check_every = 15  # seconds
+        for _ in xrange(wait_for / check_every):
+            time.sleep(check_every)
+
+            response, content = http.request(path, 'HEAD')
+            if (response['x-image-meta-status'] == 'deleted' and
+                    response['x-image-meta-deleted'] == 'True'):
+                break
+            else:
+                continue
         else:
-            continue
-    else:
-        self.fail('image was never scrubbed')
+            self.fail('image was never scrubbed')
