@@ -1,3 +1,5 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright 2012 Red Hat, Inc
 # All Rights Reserved.
 #
@@ -17,9 +19,7 @@
 
 
 from glance.tests import functional
-from glance.tests.utils import depends_on_exe
-from glance.tests.utils import execute
-from glance.tests.utils import skip_if_disabled
+from glance.tests.utils import execute, depends_on_exe, skip_if_disabled
 
 
 class TestSqlite(functional.FunctionalTest):
@@ -35,6 +35,6 @@ class TestSqlite(functional.FunctionalTest):
         cmd = "sqlite3 tests.sqlite '.schema'"
         exitcode, out, err = execute(cmd, raise_error=True)
 
-        self.assertNotIn('BIGINT', out)
+        self.assertFalse('BIGINT' in out)
 
         self.stop_servers()
