@@ -75,9 +75,6 @@ class TestCopyToFile(functional.FunctionalTest):
 
         self.start_servers(**self.__dict__.copy())
 
-        api_port = self.api_port
-        registry_port = self.registry_port
-
         # POST /images with public image to be stored in from_store,
         # to stand in for the 'external' image
         image_data = "*" * FIVE_KB
@@ -116,6 +113,19 @@ class TestCopyToFile(functional.FunctionalTest):
         # GET image and make sure image content is as expected
         path = "http://%s:%d/v1/images/%s" % ("127.0.0.1", self.api_port,
                                               copy_image_id)
+
+        def _await_status(expected_status):
+            for i in xrange(100):
+                time.sleep(0.01)
+                http = httplib2.Http()
+                response, content = http.request(path, 'HEAD')
+                self.assertEqual(response.status, 200)
+                if response['x-image-meta-status'] == expected_status:
+                    return
+            self.fail('unexpected image status %s' %
+                      response['x-image-meta-status'])
+        _await_status('active')
+
         http = httplib2.Http()
         response, content = http.request(path, 'GET')
         self.assertEqual(response.status, 200)
@@ -187,9 +197,6 @@ class TestCopyToFile(functional.FunctionalTest):
 
         setup_http(self)
 
-        api_port = self.api_port
-        registry_port = self.registry_port
-
         uri = get_http_uri(self, 'foobar')
         copy_from = uri if exists else uri.replace('images', 'snafu')
 
@@ -259,9 +266,6 @@ class TestCopyToFile(functional.FunctionalTest):
 
         self.start_servers(**self.__dict__.copy())
 
-        api_port = self.api_port
-        registry_port = self.registry_port
-
         with tempfile.NamedTemporaryFile() as image_file:
             image_file.write("XXX")
             image_file.flush()
