@@ -18,6 +18,7 @@
 import time
 
 import httplib2
+import psutil
 
 from glance.tests import functional
 from glance.tests.utils import execute
@@ -43,12 +44,12 @@ class TestMultiprocessing(functional.FunctionalTest):
         self.stop_servers()
 
     def _get_children(self):
-        api_pid = open(self.api_server.pid_file).read().strip()
-        cmd = ("ps --no-headers --ppid %s -o pid,cmd | "
-               "grep python | "  # NOTE(markwash): ignore non-python procs
-               "awk '{print $1; print >> \"/dev/stderr\"}'" % api_pid)
-        _, out, err = execute(cmd, raise_error=True)
-        return out.strip().split('\n')
+        api_pid = self.api_server.process_pid
+        process = psutil.Process(api_pid)
+
+        children = process.get_children()
+        pids = [str(child.pid) for child in children]
+        return pids
 
     def test_interrupt_avoids_respawn_storm(self):
         """
