@@ -1,3 +1,4 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
 # -*- coding: utf-8 -*-
 
 # Copyright 2013 Red Hat, Inc.
@@ -14,21 +15,22 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import json
 import datetime
 
-from oslo_serialization import jsonutils
-from oslo_utils import encodeutils
+import mox
+from oslo.config import cfg
 import routes
-import six
-from six.moves import http_client as http
 import webob
 
-from glance.common import exception
 from glance.common import rpc
 from glance.common import wsgi
+from glance.common import exception
 from glance.tests.unit import base
 from glance.tests import utils as test_utils
 
+CONF = cfg.CONF
+
 
 class FakeResource(object):
     """
@@ -76,6 +78,9 @@ class TestRPCController(base.IsolatedUnitTest):
         self.controller = rpc.Controller()
         self.controller.register(self.res)
 
+        # Mock
+        self.mocker = mox.Mox()
+
     def test_register(self):
         res = FakeResource()
         controller = rpc.Controller()
@@ -100,7 +105,7 @@ class TestRPCController(base.IsolatedUnitTest):
         controller = rpc.Controller()
 
         # Not callable
-        self.assertRaises(TypeError,
+        self.assertRaises(AssertionError,
                           controller.register,
                           res, refiner="get_all_images")
 
@@ -118,22 +123,22 @@ class TestRPCController(base.IsolatedUnitTest):
         api = create_api()
         req = webob.Request.blank('/rpc')
         req.method = 'POST'
-        req.body = jsonutils.dump_as_bytes([
+        req.body = json.dumps([
             {
                 "command": "get_images",
                 "kwargs": {"keyword": 1}
             }
         ])
         res = req.get_response(api)
-        returned = jsonutils.loads(res.body)
-        self.assertIsInstance(returned, list)
-        self.assertEqual(1, returned[0])
+        returned = json.loads(res.body)
+        self.assertTrue(isinstance(returned, list))
+        self.assertEquals(returned[0], 1)
 
     def test_request_exc(self):
         api = create_api()
         req = webob.Request.blank('/rpc')
         req.method = 'POST'
-        req.body = jsonutils.dump_as_bytes([
+        req.body = json.dumps([
             {
                 "command": "get_all_images",
                 "kwargs": {"keyword": 1}
@@ -143,7 +148,7 @@ class TestRPCController(base.IsolatedUnitTest):
         # Sending non-accepted keyword
         # to get_all_images method
         res = req.get_response(api)
-        returned = jsonutils.loads(res.body)
+        returned = json.loads(res.body)
         self.assertIn("_error", returned[0])
 
     def test_rpc_errors(self):
@@ -153,29 +158,29 @@ class TestRPCController(base.IsolatedUnitTest):
         req.content_type = 'application/json'
 
         # Body is not a list, it should fail
-        req.body = jsonutils.dump_as_bytes({})
+        req.body = json.dumps({})
         res = req.get_response(api)
-        self.assertEqual(http.BAD_REQUEST, res.status_int)
+        self.assertEquals(res.status_int, 400)
 
         # cmd is not dict, it should fail.
-        req.body = jsonutils.dump_as_bytes([None])
+        req.body = json.dumps([None])
         res = req.get_response(api)
-        self.assertEqual(http.BAD_REQUEST, res.status_int)
+        self.assertEquals(res.status_int, 400)
 
         # No command key, it should fail.
-        req.body = jsonutils.dump_as_bytes([{}])
+        req.body = json.dumps([{}])
         res = req.get_response(api)
-        self.assertEqual(http.BAD_REQUEST, res.status_int)
+        self.assertEquals(res.status_int, 400)
 
         # kwargs not dict, it should fail.
-        req.body = jsonutils.dump_as_bytes([{"command": "test", "kwargs": 2}])
+        req.body = json.dumps([{"command": "test", "kwargs": 200}])
         res = req.get_response(api)
-        self.assertEqual(http.BAD_REQUEST, res.status_int)
+        self.assertEquals(res.status_int, 400)
 
         # Command does not exist, it should fail.
-        req.body = jsonutils.dump_as_bytes([{"command": "test"}])
+        req.body = json.dumps([{"command": "test"}])
         res = req.get_response(api)
-        self.assertEqual(http.NOT_FOUND, res.status_int)
+        self.assertEquals(res.status_int, 404)
 
     def test_rpc_exception_propagation(self):
         api = create_api()
@@ -183,21 +188,20 @@ class TestRPCController(base.IsolatedUnitTest):
         req.method = 'POST'
         req.content_type = 'application/json'
 
-        req.body = jsonutils.dump_as_bytes([{"command": "raise_value_error"}])
+        req.body = json.dumps([{"command": "raise_value_error"}])
         res = req.get_response(api)
-        self.assertEqual(http.OK, res.status_int)
+        self.assertEquals(res.status_int, 200)
 
-        returned = jsonutils.loads(res.body)[0]
-        err_cls = 'builtins.ValueError' if six.PY3 else 'exceptions.ValueError'
-        self.assertEqual(err_cls, returned['_error']['cls'])
+        returned = json.loads(res.body)[0]
+        self.assertEquals(returned['_error']['cls'], 'exceptions.ValueError')
 
-        req.body = jsonutils.dump_as_bytes([{"command": "raise_weird_error"}])
+        req.body = json.dumps([{"command": "raise_weird_error"}])
         res = req.get_response(api)
-        self.assertEqual(http.OK, res.status_int)
+        self.assertEquals(res.status_int, 200)
 
-        returned = jsonutils.loads(res.body)[0]
-        self.assertEqual('glance.common.exception.RPCError',
-                         returned['_error']['cls'])
+        returned = json.loads(res.body)[0]
+        self.assertEquals(returned['_error']['cls'],
+                          'glance.common.exception.RPCError')
 
 
 class TestRPCClient(base.IsolatedUnitTest):
@@ -210,7 +214,6 @@ class TestRPCClient(base.IsolatedUnitTest):
 
     def fake_request(self, method, url, body, headers):
         req = webob.Request.blank(url.path)
-        body = encodeutils.to_utf8(body)
         req.body = body
         req.method = method
 
@@ -228,7 +231,7 @@ class TestRPCClient(base.IsolatedUnitTest):
                     {"command": "get_all_images"}]
 
         res = self.client.bulk_request(commands)
-        self.assertEqual(2, len(res))
+        self.assertEquals(len(res), 2)
         self.assertTrue(res[0])
         self.assertFalse(res[1])
 
@@ -237,7 +240,7 @@ class TestRPCClient(base.IsolatedUnitTest):
             self.client.raise_value_error()
             self.fail("Exception not raised")
         except ValueError as exc:
-            self.assertEqual("Yep, Just like that!", str(exc))
+            self.assertEquals(str(exc), "Yep, Just like that!")
 
     def test_rpc_exception(self):
         try:
@@ -248,46 +251,41 @@ class TestRPCClient(base.IsolatedUnitTest):
 
     def test_non_str_or_dict_response(self):
         rst = self.client.count_images(images=[1, 2, 3, 4])
-        self.assertEqual(4, rst)
-        self.assertIsInstance(rst, int)
+        self.assertEquals(rst, 4)
+        self.assertTrue(isinstance(rst, int))
 
 
 class TestRPCJSONSerializer(test_utils.BaseTestCase):
 
     def test_to_json(self):
         fixture = {"key": "value"}
-        expected = b'{"key": "value"}'
+        expected = '{"key": "value"}'
         actual = rpc.RPCJSONSerializer().to_json(fixture)
-        self.assertEqual(expected, actual)
+        self.assertEqual(actual, expected)
 
     def test_to_json_with_date_format_value(self):
         fixture = {"date": datetime.datetime(1900, 3, 8, 2)}
-        expected = {"date": {"_value": "1900-03-08T02:00:00",
-                             "_type": "datetime"}}
+        expected = ('{"date": {"_value": "1900-03-08T02:00:00.000000", '
+                    '"_type": "datetime"}}')
         actual = rpc.RPCJSONSerializer().to_json(fixture)
-        actual = jsonutils.loads(actual)
-        for k in expected['date']:
-            self.assertEqual(expected['date'][k], actual['date'][k])
+        self.assertEqual(actual, expected)
 
     def test_to_json_with_more_deep_format(self):
         fixture = {"is_public": True, "name": [{"name1": "test"}]}
-        expected = {"is_public": True, "name": [{"name1": "test"}]}
+        expected = '{"is_public": true, "name": [{"name1": "test"}]}'
         actual = rpc.RPCJSONSerializer().to_json(fixture)
-        actual = wsgi.JSONResponseSerializer().to_json(fixture)
-        actual = jsonutils.loads(actual)
-        for k in expected:
-            self.assertEqual(expected[k], actual[k])
+        self.assertEqual(actual, expected)
 
     def test_default(self):
         fixture = {"key": "value"}
         response = webob.Response()
         rpc.RPCJSONSerializer().default(response, fixture)
-        self.assertEqual(http.OK, response.status_int)
-        content_types = [h for h in response.headerlist
-                         if h[0] == 'Content-Type']
-        self.assertEqual(1, len(content_types))
-        self.assertEqual('application/json', response.content_type)
-        self.assertEqual(b'{"key": "value"}', response.body)
+        self.assertEqual(response.status_int, 200)
+        content_types = filter(lambda h: h[0] == 'Content-Type',
+                               response.headerlist)
+        self.assertEqual(len(content_types), 1)
+        self.assertEqual(response.content_type, 'application/json')
+        self.assertEqual(response.body, '{"key": "value"}')
 
 
 class TestRPCJSONDeserializer(test_utils.BaseTestCase):
@@ -295,22 +293,22 @@ class TestRPCJSONDeserializer(test_utils.BaseTestCase):
     def test_has_body_no_content_length(self):
         request = wsgi.Request.blank('/')
         request.method = 'POST'
-        request.body = b'asdf'
+        request.body = 'asdf'
         request.headers.pop('Content-Length')
         self.assertFalse(rpc.RPCJSONDeserializer().has_body(request))
 
     def test_has_body_zero_content_length(self):
         request = wsgi.Request.blank('/')
         request.method = 'POST'
-        request.body = b'asdf'
+        request.body = 'asdf'
         request.headers['Content-Length'] = 0
         self.assertFalse(rpc.RPCJSONDeserializer().has_body(request))
 
     def test_has_body_has_content_length(self):
         request = wsgi.Request.blank('/')
         request.method = 'POST'
-        request.body = b'asdf'
-        self.assertIn('Content-Length', request.headers)
+        request.body = 'asdf'
+        self.assertTrue('Content-Length' in request.headers)
         self.assertTrue(rpc.RPCJSONDeserializer().has_body(request))
 
     def test_no_body_no_content_length(self):
@@ -321,7 +319,7 @@ class TestRPCJSONDeserializer(test_utils.BaseTestCase):
         fixture = '{"key": "value"}'
         expected = {"key": "value"}
         actual = rpc.RPCJSONDeserializer().from_json(fixture)
-        self.assertEqual(expected, actual)
+        self.assertEqual(actual, expected)
 
     def test_from_json_malformed(self):
         fixture = 'kjasdklfjsklajf'
@@ -332,22 +330,22 @@ class TestRPCJSONDeserializer(test_utils.BaseTestCase):
         request = wsgi.Request.blank('/')
         actual = rpc.RPCJSONDeserializer().default(request)
         expected = {}
-        self.assertEqual(expected, actual)
+        self.assertEqual(actual, expected)
 
     def test_default_with_body(self):
         request = wsgi.Request.blank('/')
         request.method = 'POST'
-        request.body = b'{"key": "value"}'
+        request.body = '{"key": "value"}'
         actual = rpc.RPCJSONDeserializer().default(request)
         expected = {"body": {"key": "value"}}
-        self.assertEqual(expected, actual)
+        self.assertEqual(actual, expected)
 
     def test_has_body_has_transfer_encoding(self):
         request = wsgi.Request.blank('/')
         request.method = 'POST'
-        request.body = b'fake_body'
-        request.headers['transfer-encoding'] = ''
-        self.assertIn('transfer-encoding', request.headers)
+        request.body = 'fake_body'
+        request.headers['transfer-encoding'] = 0
+        self.assertTrue('transfer-encoding' in request.headers)
         self.assertTrue(rpc.RPCJSONDeserializer().has_body(request))
 
     def test_to_json_with_date_format_value(self):
@@ -355,4 +353,4 @@ class TestRPCJSONDeserializer(test_utils.BaseTestCase):
                    '"_type": "datetime"}}')
         expected = {"date": datetime.datetime(1900, 3, 8, 2)}
         actual = rpc.RPCJSONDeserializer().from_json(fixture)
-        self.assertEqual(expected, actual)
+        self.assertEqual(actual, expected)
