@@ -1,5 +1,4 @@
 # Copyright 2013 OpenStack Foundation.
-# Copyright 2013 IBM Corp.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,8 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from unittest import mock
-
 from glance.domain import proxy
 import glance.tests.utils as test_utils
 
@@ -47,8 +44,6 @@ class FakeRepo(object):
     add = fake_method
     save = fake_method
     remove = fake_method
-    set_property_atomic = fake_method
-    delete_property_atomic = fake_method
 
 
 class TestProxyRepoPlain(test_utils.BaseTestCase):
@@ -61,9 +56,9 @@ class TestProxyRepoPlain(test_utils.BaseTestCase):
         self.fake_repo.result = base_result
         method = getattr(self.proxy_repo, name)
         proxy_result = method(*args, **kwargs)
-        self.assertEqual(base_result, proxy_result)
-        self.assertEqual(args, self.fake_repo.args)
-        self.assertEqual(kwargs, self.fake_repo.kwargs)
+        self.assertEqual(proxy_result, base_result)
+        self.assertEqual(self.fake_repo.args, args)
+        self.assertEqual(self.fake_repo.kwargs, kwargs)
 
     def test_get(self):
         self._test_method('get', 'snarf', 'abcd')
@@ -75,33 +70,11 @@ class TestProxyRepoPlain(test_utils.BaseTestCase):
         self._test_method('add', 'snuff', 'enough')
 
     def test_save(self):
-        self._test_method('save', 'snuff', 'enough', from_state=None)
+        self._test_method('save', 'snuff', 'enough')
 
     def test_remove(self):
         self._test_method('add', None, 'flying')
 
-    def test_set_property_atomic(self):
-        image = mock.MagicMock()
-        image.image_id = 'foo'
-        self._test_method('set_property_atomic', None, image, 'foo', 'bar')
-
-    def test_set_property_nonimage(self):
-        self.assertRaises(
-            AssertionError,
-            self._test_method,
-            'set_property_atomic', None, 'notimage', 'foo', 'bar')
-
-    def test_delete_property_atomic(self):
-        image = mock.MagicMock()
-        image.image_id = 'foo'
-        self._test_method('delete_property_atomic', None, image, 'foo', 'bar')
-
-    def test_delete_property_nonimage(self):
-        self.assertRaises(
-            AssertionError,
-            self._test_method,
-            'delete_property_atomic', None, 'notimage', 'foo', 'bar')
-
 
 class TestProxyRepoWrapping(test_utils.BaseTestCase):
     def setUp(self):
@@ -115,51 +88,51 @@ class TestProxyRepoWrapping(test_utils.BaseTestCase):
         self.fake_repo.result = base_result
         method = getattr(self.proxy_repo, name)
         proxy_result = method(*args, **kwargs)
-        self.assertIsInstance(proxy_result, FakeProxy)
-        self.assertEqual(base_result, proxy_result.base)
-        self.assertEqual(0, len(proxy_result.args))
-        self.assertEqual({'a': 1}, proxy_result.kwargs)
-        self.assertEqual(args, self.fake_repo.args)
-        self.assertEqual(kwargs, self.fake_repo.kwargs)
+        self.assertTrue(isinstance(proxy_result, FakeProxy))
+        self.assertEquals(proxy_result.base, base_result)
+        self.assertEquals(len(proxy_result.args), 0)
+        self.assertEquals(proxy_result.kwargs, {'a': 1})
+        self.assertEqual(self.fake_repo.args, args)
+        self.assertEqual(self.fake_repo.kwargs, kwargs)
 
     def test_get(self):
         self.fake_repo.result = 'snarf'
         result = self.proxy_repo.get('some-id')
-        self.assertIsInstance(result, FakeProxy)
-        self.assertEqual(('some-id',), self.fake_repo.args)
-        self.assertEqual({}, self.fake_repo.kwargs)
-        self.assertEqual('snarf', result.base)
-        self.assertEqual(tuple(), result.args)
-        self.assertEqual({'a': 1}, result.kwargs)
+        self.assertTrue(isinstance(result, FakeProxy))
+        self.assertEqual(self.fake_repo.args, ('some-id',))
+        self.assertEqual(self.fake_repo.kwargs, {})
+        self.assertEqual(result.base, 'snarf')
+        self.assertEqual(result.args, tuple())
+        self.assertEqual(result.kwargs, {'a': 1})
 
     def test_list(self):
         self.fake_repo.result = ['scratch', 'sniff']
         results = self.proxy_repo.list(2, prefix='s')
-        self.assertEqual((2,), self.fake_repo.args)
-        self.assertEqual({'prefix': 's'}, self.fake_repo.kwargs)
-        self.assertEqual(2, len(results))
-        for i in range(2):
-            self.assertIsInstance(results[i], FakeProxy)
-            self.assertEqual(self.fake_repo.result[i], results[i].base)
-            self.assertEqual(tuple(), results[i].args)
-            self.assertEqual({'a': 1}, results[i].kwargs)
+        self.assertEqual(self.fake_repo.args, (2,))
+        self.assertEqual(self.fake_repo.kwargs, {'prefix': 's'})
+        self.assertEqual(len(results), 2)
+        for i in xrange(2):
+            self.assertTrue(isinstance(results[i], FakeProxy))
+            self.assertEqual(results[i].base, self.fake_repo.result[i])
+            self.assertEqual(results[i].args, tuple())
+            self.assertEqual(results[i].kwargs, {'a': 1})
 
-    def _test_method_with_proxied_argument(self, name, result, **kwargs):
+    def _test_method_with_proxied_argument(self, name, result):
         self.fake_repo.result = result
         item = FakeProxy('snoop')
         method = getattr(self.proxy_repo, name)
         proxy_result = method(item)
 
-        self.assertEqual(('snoop',), self.fake_repo.args)
-        self.assertEqual(kwargs, self.fake_repo.kwargs)
+        self.assertEqual(self.fake_repo.args, ('snoop',))
+        self.assertEqual(self.fake_repo.kwargs, {})
 
         if result is None:
-            self.assertIsNone(proxy_result)
+            self.assertTrue(proxy_result is None)
         else:
-            self.assertIsInstance(proxy_result, FakeProxy)
-            self.assertEqual(result, proxy_result.base)
-            self.assertEqual(tuple(), proxy_result.args)
-            self.assertEqual({'a': 1}, proxy_result.kwargs)
+            self.assertTrue(isinstance(proxy_result, FakeProxy))
+            self.assertEqual(proxy_result.base, result)
+            self.assertEqual(proxy_result.args, tuple())
+            self.assertEqual(proxy_result.kwargs, {'a': 1})
 
     def test_add(self):
         self._test_method_with_proxied_argument('add', 'dog')
@@ -168,12 +141,10 @@ class TestProxyRepoWrapping(test_utils.BaseTestCase):
         self._test_method_with_proxied_argument('add', None)
 
     def test_save(self):
-        self._test_method_with_proxied_argument('save', 'dog',
-                                                from_state=None)
+        self._test_method_with_proxied_argument('save', 'dog')
 
     def test_save_with_no_result(self):
-        self._test_method_with_proxied_argument('save', None,
-                                                from_state=None)
+        self._test_method_with_proxied_argument('save', None)
 
     def test_remove(self):
         self._test_method_with_proxied_argument('remove', 'dog')
@@ -201,8 +172,8 @@ class TestImageFactory(test_utils.BaseTestCase):
         proxy_factory = proxy.ImageFactory(self.factory)
         self.factory.result = 'eddard'
         image = proxy_factory.new_image(a=1, b='two')
-        self.assertEqual('eddard', image)
-        self.assertEqual({'a': 1, 'b': 'two'}, self.factory.kwargs)
+        self.assertEqual(image, 'eddard')
+        self.assertEqual(self.factory.kwargs, {'a': 1, 'b': 'two'})
 
     def test_proxy_wrapping(self):
         proxy_factory = proxy.ImageFactory(self.factory,
@@ -210,9 +181,9 @@ class TestImageFactory(test_utils.BaseTestCase):
                                            proxy_kwargs={'dog': 'bark'})
         self.factory.result = 'stark'
         image = proxy_factory.new_image(a=1, b='two')
-        self.assertIsInstance(image, FakeProxy)
-        self.assertEqual('stark', image.base)
-        self.assertEqual({'a': 1, 'b': 'two'}, self.factory.kwargs)
+        self.assertTrue(isinstance(image, FakeProxy))
+        self.assertEqual(image.base, 'stark')
+        self.assertEqual(self.factory.kwargs, {'a': 1, 'b': 'two'})
 
 
 class FakeImageMembershipFactory(object):
@@ -236,91 +207,75 @@ class TestImageMembershipFactory(test_utils.BaseTestCase):
         proxy_factory = proxy.ImageMembershipFactory(self.factory)
         self.factory.result = 'tyrion'
         membership = proxy_factory.new_image_member('jaime', 'cersei')
-        self.assertEqual('tyrion', membership)
-        self.assertEqual('jaime', self.factory.image)
-        self.assertEqual('cersei', self.factory.member_id)
+        self.assertEqual(membership, 'tyrion')
+        self.assertEqual(self.factory.image, 'jaime')
+        self.assertEqual(self.factory.member_id, 'cersei')
 
     def test_proxy_wrapped_membership(self):
         proxy_factory = proxy.ImageMembershipFactory(
-            self.factory, proxy_class=FakeProxy, proxy_kwargs={'a': 1})
+                self.factory, member_proxy_class=FakeProxy,
+                member_proxy_kwargs={'a': 1})
         self.factory.result = 'tyrion'
         membership = proxy_factory.new_image_member('jaime', 'cersei')
-        self.assertIsInstance(membership, FakeProxy)
-        self.assertEqual('tyrion', membership.base)
-        self.assertEqual({'a': 1}, membership.kwargs)
-        self.assertEqual('jaime', self.factory.image)
-        self.assertEqual('cersei', self.factory.member_id)
+        self.assertTrue(isinstance(membership, FakeProxy))
+        self.assertEqual(membership.base, 'tyrion')
+        self.assertEqual(membership.kwargs, {'a': 1})
+        self.assertEqual(self.factory.image, 'jaime')
+        self.assertEqual(self.factory.member_id, 'cersei')
 
     def test_proxy_wrapped_image(self):
         proxy_factory = proxy.ImageMembershipFactory(
-            self.factory, proxy_class=FakeProxy)
+                self.factory, image_proxy_class=FakeProxy)
         self.factory.result = 'tyrion'
         image = FakeProxy('jaime')
         membership = proxy_factory.new_image_member(image, 'cersei')
-        self.assertIsInstance(membership, FakeProxy)
-        self.assertIsInstance(self.factory.image, FakeProxy)
-        self.assertEqual('cersei', self.factory.member_id)
+        self.assertEqual(membership, 'tyrion')
+        self.assertEqual(self.factory.image, 'jaime')
+        self.assertEqual(self.factory.member_id, 'cersei')
 
     def test_proxy_both_wrapped(self):
         class FakeProxy2(FakeProxy):
             pass
 
         proxy_factory = proxy.ImageMembershipFactory(
-            self.factory,
-            proxy_class=FakeProxy,
-            proxy_kwargs={'b': 2})
+                self.factory,
+                member_proxy_class=FakeProxy,
+                member_proxy_kwargs={'b': 2},
+                image_proxy_class=FakeProxy2)
 
         self.factory.result = 'tyrion'
         image = FakeProxy2('jaime')
         membership = proxy_factory.new_image_member(image, 'cersei')
-        self.assertIsInstance(membership, FakeProxy)
-        self.assertEqual('tyrion', membership.base)
-        self.assertEqual({'b': 2}, membership.kwargs)
-        self.assertIsInstance(self.factory.image, FakeProxy2)
-        self.assertEqual('cersei', self.factory.member_id)
+        self.assertTrue(isinstance(membership, FakeProxy))
+        self.assertEqual(membership.base, 'tyrion')
+        self.assertEqual(membership.kwargs, {'b': 2})
+        self.assertEqual(self.factory.image, 'jaime')
+        self.assertEqual(self.factory.member_id, 'cersei')
 
 
 class FakeImage(object):
     def __init__(self, result=None):
         self.result = result
 
+    def get_member_repo(self):
+        return self.result
 
-class TestTaskFactory(test_utils.BaseTestCase):
+
+class TestImage(test_utils.BaseTestCase):
     def setUp(self):
-        super(TestTaskFactory, self).setUp()
-        self.factory = mock.Mock()
-        self.fake_type = 'import'
-        self.fake_owner = "owner"
+        super(TestImage, self).setUp()
+        self.image = FakeImage()
 
-    def test_proxy_plain(self):
-        proxy_factory = proxy.TaskFactory(self.factory)
+    def test_normal_member_repo(self):
+        proxy_image = proxy.Image(self.image)
+        self.image.result = 'mormont'
+        self.assertEqual(proxy_image.get_member_repo(), 'mormont')
 
-        proxy_factory.new_task(
-            type=self.fake_type,
-            owner=self.fake_owner
-        )
-
-        self.factory.new_task.assert_called_once_with(
-            type=self.fake_type,
-            owner=self.fake_owner
-        )
-
-    def test_proxy_wrapping(self):
-        proxy_factory = proxy.TaskFactory(
-            self.factory,
-            task_proxy_class=FakeProxy,
-            task_proxy_kwargs={'dog': 'bark'})
-
-        self.factory.new_task.return_value = 'fake_task'
-
-        task = proxy_factory.new_task(
-            type=self.fake_type,
-            owner=self.fake_owner
-        )
-
-        self.factory.new_task.assert_called_once_with(
-            type=self.fake_type,
-            owner=self.fake_owner
-        )
-        self.assertIsInstance(task, FakeProxy)
-        self.assertEqual('fake_task', task.base)
+    def test_proxied_member_repo(self):
+        proxy_image = proxy.Image(self.image,
+                                  member_repo_proxy_class=FakeProxy,
+                                  member_repo_proxy_kwargs={'a': 10})
+        self.image.result = 'corn'
+        member_repo = proxy_image.get_member_repo()
+        self.assertTrue(isinstance(member_repo, FakeProxy))
+        self.assertEqual(member_repo.base, 'corn')
