@@ -16,9 +16,12 @@
 #    under the License.
 
 from contextlib import contextmanager
+import datetime
 import hashlib
 import os
 import StringIO
+import tempfile
+import time
 
 import fixtures
 import stubout
@@ -27,6 +30,8 @@ from glance.common import exception
 from glance import image_cache
 #NOTE(bcwaldon): This is imported to load the registry config options
 import glance.registry
+import glance.store.filesystem as fs_store
+import glance.store.s3 as s3_store
 from glance.tests import utils as test_utils
 from glance.tests.utils import skip_if_disabled, xattr_writes_supported
 
@@ -144,6 +149,35 @@ class ImageCacheTestCase(object):
 
         self.assertFalse(os.path.exists(incomplete_file_path))
 
+    @skip_if_disabled
+    def test_clean_stalled_nonzero_stall_time(self):
+        """
+        Test the clean method removes the stalled images as expected
+        """
+        incomplete_file_path_1 = os.path.join(self.cache_dir,
+                                              'incomplete', '1')
+        incomplete_file_path_2 = os.path.join(self.cache_dir,
+                                              'incomplete', '2')
+        for f in (incomplete_file_path_1, incomplete_file_path_2):
+            incomplete_file = open(f, 'w')
+            incomplete_file.write(FIXTURE_DATA)
+            incomplete_file.close()
+
+        mtime = os.path.getmtime(incomplete_file_path_1)
+        pastday = datetime.datetime.fromtimestamp(mtime) - \
+            datetime.timedelta(days=1)
+        atime = int(time.mktime(pastday.timetuple()))
+        mtime = atime
+        os.utime(incomplete_file_path_1, (atime, mtime))
+
+        self.assertTrue(os.path.exists(incomplete_file_path_1))
+        self.assertTrue(os.path.exists(incomplete_file_path_2))
+
+        self.cache.clean(stall_time=3600)
+
+        self.assertFalse(os.path.exists(incomplete_file_path_1))
+        self.assertTrue(os.path.exists(incomplete_file_path_2))
+
     @skip_if_disabled
     def test_prune(self):
         """
@@ -384,6 +418,44 @@ class ImageCacheTestCase(object):
         # checksum is valid, fake image should be cached:
         self.assertTrue(cache.is_cached(image_id))
 
+    def test_gate_caching_iter_fs_chunked_file(self):
+        """Tests get_caching_iter when using a filesystem ChunkedFile"""
+        image_id = 123
+
+        with tempfile.NamedTemporaryFile() as test_data_file:
+            test_data_file.write(FIXTURE_DATA)
+            test_data_file.seek(0)
+            image = fs_store.ChunkedFile(test_data_file.name)
+            md5 = hashlib.md5()
+            md5.update(FIXTURE_DATA)
+            checksum = md5.hexdigest()
+
+            cache = image_cache.ImageCache()
+            img_iter = cache.get_caching_iter(image_id, checksum, image)
+            for chunk in img_iter:
+                pass
+            # checksum is valid, fake image should be cached:
+            self.assertTrue(cache.is_cached(image_id))
+
+    def test_gate_caching_iter_s3_chunked_file(self):
+        """Tests get_caching_iter when using an S3 ChunkedFile"""
+        image_id = 123
+
+        with tempfile.NamedTemporaryFile() as test_data_file:
+            test_data_file.write(FIXTURE_DATA)
+            test_data_file.seek(0)
+            image = s3_store.ChunkedFile(test_data_file)
+            md5 = hashlib.md5()
+            md5.update(FIXTURE_DATA)
+            checksum = md5.hexdigest()
+
+            cache = image_cache.ImageCache()
+            img_iter = cache.get_caching_iter(image_id, checksum, image)
+            for chunk in img_iter:
+                pass
+            # checksum is valid, fake image should be cached:
+            self.assertTrue(cache.is_cached(image_id))
+
     def test_gate_caching_iter_bad_checksum(self):
         image = "12345678990abcdefghijklmnop"
         image_id = 123
