@@ -13,13 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo.config import cfg
+
 from glance.common import exception
 from glance.common import wsgi
 import glance.context
 import glance.db.simple.api as simple_db
 import glance.openstack.common.log as logging
+import glance.store
 
 
+CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 UUID1 = 'c80a1a6c-bd1f-41c5-90ee-81afedb1d58d'
@@ -36,14 +40,14 @@ BASE_URI = 'swift+http://storeurl.com/container'
 
 
 def get_fake_request(path='', method='POST', is_admin=False, user=USER1,
-                     tenant=TENANT1):
+                     roles=['member'], tenant=TENANT1):
     req = wsgi.Request.blank(path)
     req.method = method
 
     kwargs = {
         'user': user,
         'tenant': tenant,
-        'roles': [],
+        'roles': roles,
         'is_admin': is_admin,
     }
 
@@ -61,7 +65,8 @@ class FakeDB(object):
     def init_db():
         images = [
             {'id': UUID1, 'owner': TENANT1, 'status': 'queued',
-             'locations': ['%s/%s' % (BASE_URI, UUID1)]},
+             'locations': [{'url': '%s/%s' % (BASE_URI, UUID1),
+                            'metadata': {}}]},
             {'id': UUID2, 'owner': TENANT1, 'status': 'queued'},
         ]
         [simple_db.image_create(None, image) for image in images]
@@ -80,6 +85,7 @@ class FakeDB(object):
             'images': {},
             'members': [],
             'tags': {},
+            'locations': [],
         }
 
     def __getattr__(self, key):
@@ -87,11 +93,16 @@ class FakeDB(object):
 
 
 class FakeStoreAPI(object):
-    def __init__(self):
+    def __init__(self, store_metadata=None):
         self.data = {
             '%s/%s' % (BASE_URI, UUID1): ('XXX', 3),
+            '%s/fake_location' % (BASE_URI): ('YYY', 3)
         }
         self.acls = {}
+        if store_metadata is None:
+            self.store_metadata = {}
+        else:
+            self.store_metadata = store_metadata
 
     def create_stores(self):
         pass
@@ -106,19 +117,28 @@ class FakeStoreAPI(object):
 
     def get_from_backend(self, context, location):
         try:
+            scheme = location[:location.find('/') - 1]
+            if scheme == 'unknown':
+                raise exception.UnknownScheme(scheme=scheme)
             return self.data[location]
         except KeyError:
             raise exception.NotFound()
 
-    def safe_delete_from_backend(self, uri, context, id, **kwargs):
+    def safe_delete_from_backend(self, context, uri, id, **kwargs):
         try:
             del self.data[uri]
         except KeyError:
             pass
 
-    def schedule_delayed_delete_from_backend(self, uri, id, **kwargs):
+    def schedule_delayed_delete_from_backend(self, context, uri, id, **kwargs):
         pass
 
+    def delete_image_from_backend(self, context, store_api, image_id, uri):
+        if CONF.delayed_delete:
+            self.schedule_delayed_delete_from_backend(context, uri, image_id)
+        else:
+            self.safe_delete_from_backend(context, uri, image_id)
+
     def get_size_from_backend(self, context, location):
         return self.get_from_backend(context, location)[1]
 
@@ -138,7 +158,10 @@ class FakeStoreAPI(object):
             raise exception.StorageWriteDenied()
         self.data[image_id] = (data, size)
         checksum = 'Z'
-        return (image_id, size, checksum)
+        return (image_id, size, checksum, self.store_metadata)
+
+    def check_location_metadata(self, val, key=''):
+        glance.store.check_location_metadata(val)
 
 
 class FakePolicyEnforcer(object):
