@@ -107,7 +107,8 @@ class TestImageMembersController(test_utils.BaseTestCase):
         self.images = [
             _db_fixture(UUID1, owner=TENANT1, name='1', size=256,
                         is_public=True,
-                        locations=['%s/%s' % (BASE_URI, UUID1)]),
+                        locations=[{'url': '%s/%s' % (BASE_URI, UUID1),
+                                    'metadata': {}}]),
             _db_fixture(UUID2, owner=TENANT1, name='2', size=512),
             _db_fixture(UUID3, owner=TENANT3, name='3', size=512),
             _db_fixture(UUID4, owner=TENANT4, name='4', size=1024),
@@ -173,6 +174,20 @@ class TestImageMembersController(test_utils.BaseTestCase):
         expected = set([TENANT1])
         self.assertEqual(actual, expected)
 
+    def test_index_allowed_by_get_members_policy(self):
+        rules = {"get_members": True}
+        self.policy.set_rules(rules)
+        request = unit_test_utils.get_fake_request()
+        output = self.controller.index(request, UUID2)
+        self.assertEqual(1, len(output['members']))
+
+    def test_index_forbidden_by_get_members_policy(self):
+        rules = {"get_members": False}
+        self.policy.set_rules(rules)
+        request = unit_test_utils.get_fake_request()
+        self.assertRaises(webob.exc.HTTPForbidden, self.controller.index,
+                          request, image_id=UUID2)
+
     def test_create(self):
         request = unit_test_utils.get_fake_request()
         image_id = UUID2
@@ -182,6 +197,34 @@ class TestImageMembersController(test_utils.BaseTestCase):
         self.assertEqual(UUID2, output.image_id)
         self.assertEqual(TENANT3, output.member_id)
 
+    def test_create_allowed_by_add_policy(self):
+        rules = {"add_member": True}
+        self.policy.set_rules(rules)
+        request = unit_test_utils.get_fake_request()
+        output = self.controller.create(request, image_id=UUID2,
+                                        member_id=TENANT3)
+        self.assertEqual(UUID2, output.image_id)
+        self.assertEqual(TENANT3, output.member_id)
+
+    def test_create_forbidden_by_add_policy(self):
+        rules = {"add_member": False}
+        self.policy.set_rules(rules)
+        request = unit_test_utils.get_fake_request()
+        self.assertRaises(webob.exc.HTTPForbidden, self.controller.create,
+                          request, image_id=UUID2, member_id=TENANT3)
+
+    def test_create_duplicate_member(self):
+        request = unit_test_utils.get_fake_request()
+        image_id = UUID2
+        member_id = TENANT3
+        output = self.controller.create(request, image_id=image_id,
+                                        member_id=member_id)
+        self.assertEqual(UUID2, output.image_id)
+        self.assertEqual(TENANT3, output.member_id)
+
+        self.assertRaises(webob.exc.HTTPConflict, self.controller.create,
+                          request, image_id=image_id, member_id=member_id)
+
     def test_update_done_by_member(self):
         request = unit_test_utils.get_fake_request(tenant=TENANT4)
         image_id = UUID2
@@ -193,6 +236,25 @@ class TestImageMembersController(test_utils.BaseTestCase):
         self.assertEqual(TENANT4, output.member_id)
         self.assertEqual('accepted', output.status)
 
+    def test_update_done_by_member_forbidden_by_policy(self):
+        rules = {"modify_member": False}
+        self.policy.set_rules(rules)
+        request = unit_test_utils.get_fake_request(tenant=TENANT4)
+        self.assertRaises(webob.exc.HTTPForbidden, self.controller.update,
+                          request, image_id=UUID2, member_id=TENANT4,
+                          status='accepted')
+
+    def test_update_done_by_member_allowed_by_policy(self):
+        rules = {"modify_member": True}
+        self.policy.set_rules(rules)
+        request = unit_test_utils.get_fake_request(tenant=TENANT4)
+        output = self.controller.update(request, image_id=UUID2,
+                                        member_id=TENANT4,
+                                        status='accepted')
+        self.assertEqual(UUID2, output.image_id)
+        self.assertEqual(TENANT4, output.member_id)
+        self.assertEqual('accepted', output.status)
+
     def test_update_done_by_owner(self):
         request = unit_test_utils.get_fake_request(tenant=TENANT1)
         image_id = UUID2
@@ -230,6 +292,7 @@ class TestImageMembersController(test_utils.BaseTestCase):
         image_id = UUID2
         res = self.controller.delete(request, image_id, member_id)
         self.assertEqual(res.body, '')
+        self.assertEqual(res.status_code, 204)
         found_member = self.db.image_member_find(
                 request.context, image_id=image_id, member=member_id)
         self.assertEqual(found_member, [])
@@ -246,6 +309,30 @@ class TestImageMembersController(test_utils.BaseTestCase):
         expected = set([TENANT4])
         self.assertEqual(actual, expected)
 
+    def test_delete_allowed_by_policies(self):
+        rules = {"get_member": True, "delete_member": True}
+        self.policy.set_rules(rules)
+        request = unit_test_utils.get_fake_request(tenant=TENANT1)
+        output = self.controller.delete(request, image_id=UUID2,
+                                        member_id=TENANT4)
+        request = unit_test_utils.get_fake_request()
+        output = self.controller.index(request, UUID2)
+        self.assertEqual(0, len(output['members']))
+
+    def test_delete_forbidden_by_get_member_policy(self):
+        rules = {"get_member": False}
+        self.policy.set_rules(rules)
+        request = unit_test_utils.get_fake_request(tenant=TENANT1)
+        self.assertRaises(webob.exc.HTTPForbidden, self.controller.delete,
+                          request, UUID2, TENANT4)
+
+    def test_delete_forbidden_by_delete_member_policy(self):
+        rules = {"delete_member": False}
+        self.policy.set_rules(rules)
+        request = unit_test_utils.get_fake_request(tenant=TENANT1)
+        self.assertRaises(webob.exc.HTTPForbidden, self.controller.delete,
+                          request, UUID2, TENANT4)
+
     def test_delete_private_image(self):
         request = unit_test_utils.get_fake_request()
         self.assertRaises(webob.exc.HTTPForbidden, self.controller.delete,
@@ -377,6 +464,12 @@ class TestImagesDeserializer(test_utils.BaseTestCase):
         self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.create,
                           request)
 
+    def test_create_member_empty(self):
+        request = unit_test_utils.get_fake_request()
+        request.body = json.dumps({'member': ''})
+        self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.create,
+                          request)
+
     def test_update(self):
         request = unit_test_utils.get_fake_request()
         request.body = json.dumps({'status': 'accepted'})
