@@ -1,14 +1,14 @@
 # Copyright 2012 OpenStack Foundation.
 # All Rights Reserved.
 #
-#    Licensed under the Apache License, Version 2.0 (the 'License'); you may
+#    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
 #
 #         http://www.apache.org/licenses/LICENSE-2.0
 #
 #    Unless required by applicable law or agreed to in writing, software
-#    distributed under the License is distributed on an 'AS IS' BASIS, WITHOUT
+#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
@@ -31,7 +31,8 @@ class TestSchemasController(test_utils.BaseTestCase):
         expected = set(['status', 'name', 'tags', 'checksum', 'created_at',
                         'disk_format', 'updated_at', 'visibility', 'self',
                         'file', 'container_format', 'schema', 'id', 'size',
-                        'direct_url', 'min_ram', 'min_disk', 'protected'])
+                        'direct_url', 'min_ram', 'min_disk', 'protected',
+                        'locations'])
         self.assertEqual(set(output['properties'].keys()), expected)
 
     def test_images(self):
