@@ -20,18 +20,24 @@
 import errno
 import functools
 import os
+import shlex
+import shutil
 import socket
 import StringIO
 import subprocess
+import sys
 
+import fixtures
 from oslo.config import cfg
 import stubout
 import testtools
+import webob
 
 from glance.common import config
 from glance.common import exception
 from glance.common import wsgi
 from glance import context
+from glance.common import property_utils
 
 CONF = cfg.CONF
 
@@ -48,12 +54,33 @@ class BaseTestCase(testtools.TestCase):
         self.addCleanup(CONF.reset)
         self.stubs = stubout.StubOutForTesting()
         self.stubs.Set(exception, '_FATAL_EXCEPTION_FORMAT_ERRORS', True)
+        self.test_dir = self.useFixture(fixtures.TempDir()).path
 
     def tearDown(self):
         self.stubs.UnsetAll()
         self.stubs.SmartUnsetAll()
         super(BaseTestCase, self).tearDown()
 
+    def set_property_protections(self):
+        self.property_file = self._copy_data_file('property-protections.conf',
+                                                  self.test_dir)
+        self.config(property_protection_file=self.property_file)
+
+    def _copy_data_file(self, file_name, dst_dir):
+        src_file_name = os.path.join('glance/tests/etc', file_name)
+        shutil.copy(src_file_name, dst_dir)
+        dst_file_name = os.path.join(dst_dir, file_name)
+        return dst_file_name
+
+    def set_property_protection_rules(self, rules):
+        f = open(self.property_file, 'w')
+        for rule_key in rules.keys():
+            f.write('[%s]\n' % rule_key)
+            for operation in rules[rule_key].keys():
+                roles_str = ','.join(rules[rule_key][operation])
+                f.write('%s = %s\n' % (operation, roles_str))
+        f.close()
+
     def config(self, **kw):
         """
         Override some configuration values.
@@ -122,6 +149,76 @@ def skip_if_disabled(func):
     return wrapped
 
 
+def fork_exec(cmd,
+              exec_env=None,
+              logfile=None):
+    """
+    Execute a command using fork/exec.
+
+    This is needed for programs system executions that need path
+    searching but cannot have a shell as their parent process, for
+    example: glance-api.  When glance-api starts it sets itself as
+    the parent process for its own process group.  Thus the pid that
+    a Popen process would have is not the right pid to use for killing
+    the process group.  This patch gives the test env direct access
+    to the actual pid.
+
+    :param cmd: Command to execute as an array of arguments.
+    :param exec_env: A dictionary representing the environment with
+                     which to run the command.
+    :param logile: A path to a file which will hold the stdout/err of
+                   the child process.
+    """
+    env = os.environ.copy()
+    if exec_env is not None:
+        for env_name, env_val in exec_env.items():
+            if callable(env_val):
+                env[env_name] = env_val(env.get(env_name))
+            else:
+                env[env_name] = env_val
+
+    pid = os.fork()
+    if pid == 0:
+        if logfile:
+            fds = [1, 2]
+            with open(logfile, 'r+b') as fptr:
+                for desc in fds:  # close fds
+                    try:
+                        os.dup2(fptr.fileno(), desc)
+                    except OSError:
+                        pass
+
+        args = shlex.split(cmd)
+        os.execvpe(args[0], args, env)
+    else:
+        return pid
+
+
+def wait_for_fork(pid,
+                  raise_error=True,
+                  expected_exitcode=0):
+    """
+    Wait for a process to complete
+
+    This function will wait for the given pid to complete.  If the
+    exit code does not match that of the expected_exitcode an error
+    is raised.
+    """
+
+    rc = 0
+    try:
+        (pid, rc) = os.waitpid(pid, 0)
+        rc = os.WEXITSTATUS(rc)
+        if rc != expected_exitcode:
+            raise RuntimeError('The exit code %d is not %d'
+                               % (rc, expected_exitcode))
+    except Exception:
+        if raise_error:
+            raise
+
+    return rc
+
+
 def execute(cmd,
             raise_error=True,
             no_venv=False,
@@ -230,11 +327,20 @@ def get_unused_port():
     """
     Returns an unused port on localhost.
     """
+    port, s = get_unused_port_and_socket()
+    s.close()
+    return port
+
+
+def get_unused_port_and_socket():
+    """
+    Returns an unused port on localhost and the open socket
+    from which it was created.
+    """
     s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     s.bind(('localhost', 0))
     addr, port = s.getsockname()
-    s.close()
-    return port
+    return (port, s)
 
 
 def xattr_writes_supported(path):
@@ -259,7 +365,7 @@ def xattr_writes_supported(path):
         fake_file.flush()
     try:
         set_xattr(fake_filepath, 'hits', '1')
-    except IOError, e:
+    except IOError as e:
         if e.errno == errno.EOPNOTSUPP:
             result = False
     else:
@@ -336,3 +442,44 @@ class FakeHTTPResponse(object):
 
     def read(self, amt):
         self.data.read(amt)
+
+
+class Httplib2WsgiAdapter(object):
+    def __init__(self, app):
+        self.app = app
+
+    def request(self, uri, method="GET", body=None, headers=None):
+        req = webob.Request.blank(uri, method=method, headers=headers)
+        req.body = body
+        resp = req.get_response(self.app)
+        return Httplib2WebobResponse(resp), resp.body
+
+
+class Httplib2WebobResponse(object):
+    def __init__(self, webob_resp):
+        self.webob_resp = webob_resp
+
+    @property
+    def status(self):
+        return self.webob_resp.status_code
+
+    def __getitem__(self, key):
+        return self.webob_resp.headers[key]
+
+    def get(self, key):
+        return self.webob_resp.headers[key]
+
+
+class HttplibWsgiAdapter(object):
+    def __init__(self, app):
+        self.app = app
+        self.req = None
+
+    def request(self, method, url, body=None, headers={}):
+        self.req = webob.Request.blank(url, method=method, headers=headers)
+        self.req.body = body
+
+    def getresponse(self):
+        response = self.req.get_response(self.app)
+        return FakeHTTPResponse(response.status_code, response.headers,
+                                response.body)
